#include "Data/Monster.h"

using namespace data;
using namespace std;

/* initialize static variables */
int Monster::_saiph_difficulty_min = 0;
int Monster::_saiph_difficulty_max = 0;
int Monster::_monster_symbol_mapping[UCHAR_MAX + 1][INVERSE_BOLD_WHITE + 1] = {
	{0}
};
vector<const Monster*> Monster::_monsters;
map<const string, int> Monster::_monster_name_mapping;

/* protected constructors */
Monster::Monster(const string& name, const unsigned char& symbol, int difficulty, int move_rate, int ac, int magic_resistance, int alignment, int geno_flags, const Attack& a0, const Attack& a1, const Attack& a2, const Attack& a3, const Attack& a4, const Attack& a5, int weight, int nutrition, int extension, int sounds, int size, int resistances, int resistances_conferred, int m1, int m2, int m3, int color) : _name(name), _symbol(symbol), _difficulty(difficulty), _move_rate(move_rate), _ac(ac), _magic_resistance(magic_resistance), _alignment(alignment), _geno_flags(geno_flags), _weight(weight), _nutrition(nutrition), _extension(extension), _sounds(sounds), _size(size), _resistances(resistances), _resistances_conferred(resistances_conferred), _m1(m1), _m2(m2), _m3(m3), _color(color), _ignores_elbereth((symbol == S_HUMAN || symbol == S_ANGEL || name == "minotaur" || name == "Death" || name == "Pestilence" || name == "Famine")) {
	_attacks.push_back(a0);
	_attacks.push_back(a1);
	_attacks.push_back(a2);
	_attacks.push_back(a3);
	_attacks.push_back(a4);
	_attacks.push_back(a5);

	/* calculate the "saiph difficulty" */
	_saiph_difficulty = 0;
	if (_ignores_elbereth)
		_saiph_difficulty += 100;
	_saiph_difficulty += _move_rate * 2; // faster monsters are more dangerous
	_saiph_difficulty -= _ac * 2; // the lower ac the monster got, the more dangerous it is
	_saiph_difficulty += _magic_resistance; // higher magic resistance is bad
	_saiph_difficulty += (a0.maxDamage() + a1.maxDamage() + a2.maxDamage() + a3.maxDamage() + a4.maxDamage() + a5.maxDamage()) * 3; // hard-hitting monsters are mean
	if (m2 & M2_MAGIC)
		_saiph_difficulty += 25; // magic users can be dangerous
	if (m3 & M3_COVETOUS)
		_saiph_difficulty += 50; // covetous!
	if (m2 & M2_NASTY)
		_saiph_difficulty += 50; // nasty!
	if (m2 & M2_STRONG)
		_saiph_difficulty += 25;
	/* should look at elemental attacks, resistances, etc */
	if (_saiph_difficulty < _saiph_difficulty_min)
		_saiph_difficulty_min = _saiph_difficulty;
	if (_saiph_difficulty > _saiph_difficulty_max)
		_saiph_difficulty_max = _saiph_difficulty;
}

/* destructor */
Monster::~Monster() {
}

/* public static methods */
void Monster::init() {
	/* set all values in monster_symbol_mapping to -1 */
	for (int a = 0; a < UCHAR_MAX + 1; ++a) {
		for (int b = 0; b < INVERSE_BOLD_WHITE + 1; ++b)
			_monster_symbol_mapping[a][b] = -1;
	}
	/* push all monster data to vector */
	_monsters.push_back(new Monster("giant ant", S_ANT, 2, 18, 3, 0, 0, (G_GENO | G_SGROUP | 3), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("killer bee", S_ANT, 1, 18, -1, 0, 0, (G_GENO | G_LGROUP | 2), Attack(AT_STNG, AD_DRST, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 1, 5, 0, MS_BUZZ, MZ_TINY, MR_POISON, MR_POISON, M1_ANIMAL | M1_FLY | M1_NOHANDS | M1_POIS, M2_HOSTILE | M2_FEMALE, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("soldier ant", S_ANT, 3, 18, 3, 0, 0, (G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(AT_STNG, AD_DRST, 3, 4), Attack(), Attack(), Attack(), Attack(), 20, 5, 0, MS_SILENT, MZ_TINY, MR_POISON, MR_POISON, M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_POIS | M1_CARNIVORE, M2_HOSTILE, 0, BLUE));
	_monsters.push_back(new Monster("fire ant", S_ANT, 3, 18, 3, 10, 0, (G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(AT_BITE, AD_FIRE, 2, 4), Attack(), Attack(), Attack(), Attack(), 30, 10, 0, MS_SILENT, MZ_TINY, MR_FIRE, MR_FIRE, M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("giant beetle", S_ANT, 5, 6, 4, 0, 0, (G_GENO | 3), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_LARGE, MR_POISON, MR_POISON, M1_ANIMAL | M1_NOHANDS | M1_POIS | M1_CARNIVORE, M2_HOSTILE, 0, BLACK));
	_monsters.push_back(new Monster("queen bee", S_ANT, 9, 24, -4, 0, 0, (G_GENO | G_NOGEN), Attack(AT_STNG, AD_DRST, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1, 5, 0, MS_BUZZ, MZ_TINY, MR_POISON, MR_POISON, M1_ANIMAL | M1_FLY | M1_NOHANDS | M1_OVIPAROUS | M1_POIS, M2_HOSTILE | M2_FEMALE | M2_PRINCE, 0, MAGENTA));
	_monsters.push_back(new Monster("acid blob", S_BLOB, 1, 3, 8, 0, 0, (G_GENO | 2), Attack(AT_NONE, AD_ACID, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 10, 0, MS_SILENT, MZ_TINY, MR_SLEEP | MR_POISON | MR_ACID | MR_STONE, MR_STONE, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_ACID, M2_WANDER | M2_NEUTER, 0, GREEN));
	_monsters.push_back(new Monster("quivering blob", S_BLOB, 5, 1, 8, 0, 0, (G_GENO | 2), Attack(AT_TUCH, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 200, 100, 0, MS_SILENT, MZ_SMALL, MR_SLEEP | MR_POISON, MR_POISON, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_WANDER | M2_HOSTILE | M2_NEUTER, 0, BOLD_WHITE));
	_monsters.push_back(new Monster("gelatinous cube", S_BLOB, 6, 6, 8, 0, 0, (G_GENO | 2), Attack(AT_TUCH, AD_PLYS, 2, 4), Attack(AT_NONE, AD_PLYS, 1, 4), Attack(), Attack(), Attack(), Attack(), 600, 150, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON | MR_ACID | MR_STONE, MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_OMNIVORE | M1_ACID, M2_WANDER | M2_HOSTILE | M2_NEUTER, 0, CYAN));
	_monsters.push_back(new Monster("chickatrice", S_COCKATRICE, 4, 4, 8, 30, 0, (G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(AT_TUCH, AD_STON, 0, 0), Attack(AT_NONE, AD_STON, 0, 0), Attack(), Attack(), Attack(), 10, 10, 0, MS_HISS, MZ_TINY, MR_POISON | MR_STONE, MR_POISON | MR_STONE, M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("cockatrice", S_COCKATRICE, 5, 6, 6, 30, 0, (G_GENO | 5), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(AT_TUCH, AD_STON, 0, 0), Attack(AT_NONE, AD_STON, 0, 0), Attack(), Attack(), Attack(), 30, 30, 0, MS_HISS, MZ_SMALL, MR_POISON | MR_STONE, MR_POISON | MR_STONE, M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE | M1_OVIPAROUS, M2_HOSTILE, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("pyrolisk", S_COCKATRICE, 6, 6, 6, 30, 0, (G_GENO | 1), Attack(AT_GAZE, AD_FIRE, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_HISS, MZ_SMALL, MR_POISON | MR_FIRE, MR_POISON | MR_FIRE, M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE | M1_OVIPAROUS, M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("jackal", S_DOG, 0, 12, 7, 0, 0, (G_GENO | G_SGROUP | 3), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 250, 0, MS_BARK, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("fox", S_DOG, 0, 15, 7, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 250, 0, MS_BARK, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("coyote", S_DOG, 1, 12, 7, 0, 0, (G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 250, 0, MS_BARK, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("werejackal", S_DOG, 2, 12, 7, 10, -7, (G_NOGEN | G_NOCORPSE), Attack(AT_BITE, AD_WERE, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 250, 0, MS_BARK, MZ_SMALL, MR_POISON, 0, M1_NOHANDS | M1_POIS | M1_REGEN | M1_CARNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("little dog", S_DOG, 2, 18, 6, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 150, 150, 0, MS_BARK, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("dog", S_DOG, 4, 16, 5, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 200, 0, MS_BARK, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("large dog", S_DOG, 6, 15, 4, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 250, 0, MS_BARK, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_STRONG | M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("dingo", S_DOG, 4, 16, 5, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 200, 0, MS_BARK, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("wolf", S_DOG, 5, 12, 4, 0, 0, (G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 250, 0, MS_BARK, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("werewolf", S_DOG, 5, 12, 4, 20, -7, (G_NOGEN | G_NOCORPSE), Attack(AT_BITE, AD_WERE, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 250, 0, MS_BARK, MZ_MEDIUM, MR_POISON, 0, M1_NOHANDS | M1_POIS | M1_REGEN | M1_CARNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("warg", S_DOG, 7, 12, 4, 0, -5, (G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 850, 350, 0, MS_BARK, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("winter wolf cub", S_DOG, 5, 12, 4, 0, -5, (G_NOHELL | G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 1, 8), Attack(AT_BREA, AD_COLD, 1, 8), Attack(), Attack(), Attack(), Attack(), 250, 200, 0, MS_BARK, MZ_SMALL, MR_COLD, MR_COLD, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, CYAN));
	_monsters.push_back(new Monster("winter wolf", S_DOG, 7, 12, 4, 20, 0, (G_NOHELL | G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(AT_BREA, AD_COLD, 2, 6), Attack(), Attack(), Attack(), Attack(), 700, 300, 0, MS_BARK, MZ_LARGE, MR_COLD, MR_COLD, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, 0, CYAN));
	_monsters.push_back(new Monster("hell hound pup", S_DOG, 7, 12, 4, 20, -5, (G_HELL | G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(AT_BREA, AD_FIRE, 2, 6), Attack(), Attack(), Attack(), Attack(), 200, 200, 0, MS_BARK, MZ_SMALL, MR_FIRE, MR_FIRE, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("hell hound", S_DOG, 12, 14, 2, 20, 0, (G_HELL | G_GENO | 1), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(AT_BREA, AD_FIRE, 3, 6), Attack(), Attack(), Attack(), Attack(), 600, 300, 0, MS_BARK, MZ_MEDIUM, MR_FIRE, MR_FIRE, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("gas spore", S_EYE, 1, 3, 10, 0, 0, (G_NOCORPSE | G_GENO | 1), Attack(AT_BOOM, AD_PHYS, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_FLY | M1_BREATHLESS | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_HOSTILE | M2_NEUTER, 0, WHITE));
	_monsters.push_back(new Monster("floating eye", S_EYE, 2, 1, 9, 10, 0, (G_GENO | 5), Attack(AT_NONE, AD_PLYS, 0, 70), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_FLY | M1_AMPHIBIOUS | M1_NOLIMBS | M1_NOHEAD | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("freezing sphere", S_EYE, 6, 13, 4, 0, 0, (G_NOCORPSE | G_NOHELL | G_GENO | 2), Attack(AT_EXPL, AD_COLD, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_SMALL, MR_COLD, MR_COLD, M1_FLY | M1_BREATHLESS | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("flaming sphere", S_EYE, 6, 13, 4, 0, 0, (G_NOCORPSE | G_GENO | 2), Attack(AT_EXPL, AD_FIRE, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_SMALL, MR_FIRE, MR_FIRE, M1_FLY | M1_BREATHLESS | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("shocking sphere", S_EYE, 6, 13, 4, 0, 0, (G_NOCORPSE | G_GENO | 2), Attack(AT_EXPL, AD_ELEC, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 10, 0, MS_SILENT, MZ_SMALL, MR_ELEC, MR_ELEC, M1_FLY | M1_BREATHLESS | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BOLD_BLUE));
	_monsters.push_back(new Monster("kitten", S_FELINE, 2, 18, 6, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 150, 150, 0, MS_MEW, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_WANDER | M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("housecat", S_FELINE, 4, 16, 5, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 200, 200, 0, MS_MEW, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("jaguar", S_FELINE, 4, 15, 6, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_BITE, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), 600, 300, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("lynx", S_FELINE, 5, 15, 6, 0, 0, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_BITE, AD_PHYS, 1, 10), Attack(), Attack(), Attack(), 600, 300, 0, MS_GROWL, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("panther", S_FELINE, 5, 15, 6, 0, 0, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_BITE, AD_PHYS, 1, 10), Attack(), Attack(), Attack(), 600, 300, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("large cat", S_FELINE, 6, 15, 4, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 250, 250, 0, MS_MEW, MZ_SMALL, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_STRONG | M2_DOMESTIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("tiger", S_FELINE, 6, 12, 6, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_BITE, AD_PHYS, 1, 10), Attack(), Attack(), Attack(), 600, 300, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("gremlin", S_GREMLIN, 5, 12, 2, 25, -9, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_CURS, 0, 0), Attack(), Attack(), 100, 20, 0, MS_LAUGH, MZ_SMALL, MR_POISON, MR_POISON, M1_SWIM | M1_HUMANOID | M1_POIS, M2_STALK, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("gargoyle", S_GREMLIN, 6, 10, -4, 0, -9, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), 1000, 200, 0, MS_GRUNT, MZ_HUMAN, MR_STONE, MR_STONE, M1_HUMANOID | M1_THICK_HIDE | M1_BREATHLESS, M2_HOSTILE | M2_STRONG, 0, YELLOW));
	_monsters.push_back(new Monster("winged gargoyle", S_GREMLIN, 9, 15, -2, 0, -12, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 3, 6), Attack(AT_CLAW, AD_PHYS, 3, 6), Attack(AT_BITE, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), 1200, 300, 0, MS_GRUNT, MZ_HUMAN, MR_STONE, MR_STONE, M1_FLY | M1_HUMANOID | M1_THICK_HIDE | M1_BREATHLESS | M1_OVIPAROUS, M2_LORD | M2_HOSTILE | M2_STRONG | M2_MAGIC, 0, MAGENTA));
	_monsters.push_back(new Monster("hobbit", S_HUMANOID, 1, 9, 10, 0, 6, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 200, 0, MS_HUMANOID, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("dwarf", S_HUMANOID, 2, 6, 10, 10, 4, (G_GENO | 3), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 900, 300, 0, MS_HUMANOID, MZ_HUMAN, 0, 0, M1_TUNNEL | M1_NEEDPICK | M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_DWARF | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("bugbear", S_HUMANOID, 3, 9, 5, 0, -6, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1250, 250, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("dwarf lord", S_HUMANOID, 4, 6, 10, 10, 5, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), 900, 300, 0, MS_HUMANOID, MZ_HUMAN, 0, 0, M1_TUNNEL | M1_NEEDPICK | M1_HUMANOID | M1_OMNIVORE, M2_DWARF | M2_STRONG | M2_LORD | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("dwarf king", S_HUMANOID, 6, 6, 10, 20, 6, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), 900, 300, 0, MS_HUMANOID, MZ_HUMAN, 0, 0, M1_TUNNEL | M1_NEEDPICK | M1_HUMANOID | M1_OMNIVORE, M2_DWARF | M2_STRONG | M2_PRINCE | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("mind flayer", S_HUMANOID, 9, 12, 5, 90, -8, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(), Attack(), 1450, 400, 0, MS_HISS, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_FLY | M1_SEE_INVIS | M1_OMNIVORE, M2_HOSTILE | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("master mind flayer", S_HUMANOID, 13, 12, 0, 90, -8, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), Attack(AT_TENT, AD_DRIN, 2, 1), 1450, 400, 0, MS_HISS, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_FLY | M1_SEE_INVIS | M1_OMNIVORE, M2_HOSTILE | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("manes", S_IMP, 1, 3, 7, 0, -7, (G_GENO | G_LGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), 100, 100, 0, MS_SILENT, MZ_SMALL, MR_SLEEP | MR_POISON, 0, M1_POIS, M2_HOSTILE | M2_STALK, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("homunculus", S_IMP, 2, 12, 6, 10, -7, (G_GENO | 2), Attack(AT_BITE, AD_SLEE, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 60, 100, 0, MS_SILENT, MZ_TINY, MR_SLEEP | MR_POISON, MR_SLEEP | MR_POISON, M1_FLY | M1_POIS, M2_STALK, M3_INFRAVISIBLE | M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("imp", S_IMP, 3, 12, 2, 20, -7, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 20, 10, 0, MS_CUSS, MZ_TINY, 0, 0, M1_REGEN, M2_WANDER | M2_STALK, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("lemure", S_IMP, 3, 3, 7, 0, -7, (G_HELL | G_GENO | G_LGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 150, 100, 0, MS_SILENT, MZ_MEDIUM, MR_SLEEP | MR_POISON, MR_SLEEP, M1_POIS | M1_REGEN, M2_HOSTILE | M2_WANDER | M2_STALK | M2_NEUTER, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("quasit", S_IMP, 3, 15, 2, 20, -7, (G_GENO | 2), Attack(AT_CLAW, AD_DRDX, 1, 2), Attack(AT_CLAW, AD_DRDX, 1, 2), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), 200, 200, 0, MS_SILENT, MZ_SMALL, MR_POISON, MR_POISON, M1_REGEN, M2_STALK, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("tengu", S_IMP, 6, 13, 5, 30, 7, (G_GENO | 3), Attack(AT_BITE, AD_PHYS, 1, 7), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 200, 0, MS_SQAWK, MZ_SMALL, MR_POISON, MR_POISON, M1_TPORT | M1_TPORT_CNTRL, M2_STALK, M3_INFRAVISIBLE | M3_INFRAVISION, CYAN));
	_monsters.push_back(new Monster("blue jelly", S_JELLY, 4, 0, 8, 10, 0, (G_GENO | 2), Attack(AT_NONE, AD_COLD, 0, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 20, 0, MS_SILENT, MZ_MEDIUM, MR_COLD | MR_POISON, MR_COLD | MR_POISON, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, BLUE));
	_monsters.push_back(new Monster("spotted jelly", S_JELLY, 5, 0, 8, 10, 0, (G_GENO | 1), Attack(AT_NONE, AD_ACID, 0, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 20, 0, MS_SILENT, MZ_MEDIUM, MR_ACID | MR_STONE, 0, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_ACID | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, GREEN));
	_monsters.push_back(new Monster("ochre jelly", S_JELLY, 6, 3, 8, 20, 0, (G_GENO | 2), Attack(AT_ENGL, AD_ACID, 3, 6), Attack(AT_NONE, AD_ACID, 3, 6), Attack(), Attack(), Attack(), Attack(), 50, 20, 0, MS_SILENT, MZ_MEDIUM, MR_ACID | MR_STONE, 0, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_ACID | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("kobold", S_KOBOLD, 0, 6, 10, 0, -2, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 100, 0, MS_ORC, MZ_SMALL, MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_OMNIVORE, M2_HOSTILE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("large kobold", S_KOBOLD, 1, 6, 10, 0, -3, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 450, 150, 0, MS_ORC, MZ_SMALL, MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_OMNIVORE, M2_HOSTILE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("kobold lord", S_KOBOLD, 2, 6, 10, 0, -4, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 200, 0, MS_ORC, MZ_SMALL, MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_OMNIVORE, M2_HOSTILE | M2_LORD | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("kobold shaman", S_KOBOLD, 2, 6, 6, 10, -4, (G_GENO | 1), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), Attack(), 450, 150, 0, MS_ORC, MZ_SMALL, MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_OMNIVORE, M2_HOSTILE | M2_MAGIC, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_BLUE));
	_monsters.push_back(new Monster("leprechaun", S_LEPRECHAUN, 5, 15, 8, 20, 0, (G_GENO | 4), Attack(AT_CLAW, AD_SGLD, 1, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 60, 30, 0, MS_LAUGH, MZ_TINY, 0, 0, M1_HUMANOID | M1_TPORT, M2_HOSTILE | M2_GREEDY, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("small mimic", S_MIMIC, 7, 3, 7, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 200, 0, MS_SILENT, MZ_MEDIUM, MR_ACID, 0, M1_BREATHLESS | M1_AMORPHOUS | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOHEAD | M1_NOLIMBS | M1_THICK_HIDE | M1_CARNIVORE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("large mimic", S_MIMIC, 8, 3, 7, 10, 0, (G_GENO | 1), Attack(AT_CLAW, AD_STCK, 3, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 600, 400, 0, MS_SILENT, MZ_LARGE, MR_ACID, 0, M1_CLING | M1_BREATHLESS | M1_AMORPHOUS | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOHEAD | M1_NOLIMBS | M1_THICK_HIDE | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, 0, RED));
	_monsters.push_back(new Monster("giant mimic", S_MIMIC, 9, 3, 7, 20, 0, (G_GENO | 1), Attack(AT_CLAW, AD_STCK, 3, 6), Attack(AT_CLAW, AD_STCK, 3, 6), Attack(), Attack(), Attack(), Attack(), 800, 500, 0, MS_SILENT, MZ_LARGE, MR_ACID, 0, M1_CLING | M1_BREATHLESS | M1_AMORPHOUS | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOHEAD | M1_NOLIMBS | M1_THICK_HIDE | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, 0, MAGENTA));
	_monsters.push_back(new Monster("wood nymph", S_NYMPH, 3, 12, 9, 20, 0, (G_GENO | 2), Attack(AT_CLAW, AD_SITM, 0, 0), Attack(AT_CLAW, AD_SEDU, 0, 0), Attack(), Attack(), Attack(), Attack(), 600, 300, 0, MS_SEDUCE, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_TPORT, M2_HOSTILE | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("water nymph", S_NYMPH, 3, 12, 9, 20, 0, (G_GENO | 2), Attack(AT_CLAW, AD_SITM, 0, 0), Attack(AT_CLAW, AD_SEDU, 0, 0), Attack(), Attack(), Attack(), Attack(), 600, 300, 0, MS_SEDUCE, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_TPORT | M1_SWIM, M2_HOSTILE | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("mountain nymph", S_NYMPH, 3, 12, 9, 20, 0, (G_GENO | 2), Attack(AT_CLAW, AD_SITM, 0, 0), Attack(AT_CLAW, AD_SEDU, 0, 0), Attack(), Attack(), Attack(), Attack(), 600, 300, 0, MS_SEDUCE, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_TPORT, M2_HOSTILE | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("goblin", S_ORC, 0, 6, 10, 0, -3, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 100, 0, MS_ORC, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("hobgoblin", S_ORC, 1, 9, 10, 0, -4, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1000, 200, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("orc", S_ORC, 1, 9, 10, 0, -3, (G_GENO | G_NOGEN | G_LGROUP), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 850, 150, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("hill orc", S_ORC, 2, 9, 10, 0, -4, (G_GENO | G_LGROUP | 2), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1000, 200, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_YELLOW));
	_monsters.push_back(new Monster("Mordor orc", S_ORC, 3, 5, 10, 0, -5, (G_GENO | G_LGROUP | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1200, 200, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("Uruk-hai", S_ORC, 3, 7, 10, 0, -4, (G_GENO | G_LGROUP | 1), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1300, 300, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BLACK));
	_monsters.push_back(new Monster("orc shaman", S_ORC, 3, 9, 5, 10, -5, (G_GENO | 1), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), Attack(), 1000, 300, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_MAGIC, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_BLUE));
	_monsters.push_back(new Monster("orc-captain", S_ORC, 5, 5, 10, 0, -5, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), 1350, 350, 0, MS_ORC, MZ_HUMAN, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_ORC | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("rock piercer", S_PIERCER, 3, 1, 3, 0, 0, (G_GENO | 4), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 200, 200, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_CLING | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOLIMBS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE, 0, WHITE));
	_monsters.push_back(new Monster("iron piercer", S_PIERCER, 5, 1, 0, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 300, 0, MS_SILENT, MZ_MEDIUM, 0, 0, M1_CLING | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOLIMBS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE, 0, CYAN));
	_monsters.push_back(new Monster("glass piercer", S_PIERCER, 7, 1, 0, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 300, 0, MS_SILENT, MZ_MEDIUM, MR_ACID, 0, M1_CLING | M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOLIMBS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE, 0, BOLD_WHITE));
	_monsters.push_back(new Monster("rothe", S_QUADRUPED, 2, 9, 7, 0, 0, (G_GENO | G_SGROUP | 4), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), 400, 100, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("mumak", S_QUADRUPED, 5, 9, 0, 0, -2, (G_GENO | 1), Attack(AT_BUTT, AD_PHYS, 4, 12), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), 2500, 500, 0, MS_ROAR, MZ_LARGE, 0, 0, M1_ANIMAL | M1_THICK_HIDE | M1_NOHANDS | M1_HERBIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("leocrotta", S_QUADRUPED, 6, 18, 4, 10, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 1200, 500, 0, MS_IMITATE, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("wumpus", S_QUADRUPED, 8, 3, 2, 10, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 2500, 500, 0, MS_BURBLE, MZ_LARGE, 0, 0, M1_CLING | M1_ANIMAL | M1_NOHANDS | M1_OMNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("titanothere", S_QUADRUPED, 12, 12, 6, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 2650, 650, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_THICK_HIDE | M1_NOHANDS | M1_HERBIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("baluchitherium", S_QUADRUPED, 14, 12, 5, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 5, 4), Attack(AT_CLAW, AD_PHYS, 5, 4), Attack(), Attack(), Attack(), Attack(), 3800, 800, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_THICK_HIDE | M1_NOHANDS | M1_HERBIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("mastodon", S_QUADRUPED, 20, 12, 5, 0, 0, (G_GENO | 1), Attack(AT_BUTT, AD_PHYS, 4, 8), Attack(AT_BUTT, AD_PHYS, 4, 8), Attack(), Attack(), Attack(), Attack(), 3800, 800, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_THICK_HIDE | M1_NOHANDS | M1_HERBIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("sewer rat", S_RODENT, 0, 12, 7, 0, 0, (G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 20, 12, 0, MS_SQEEK, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("giant rat", S_RODENT, 1, 10, 7, 0, 0, (G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_SQEEK, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("rabid rat", S_RODENT, 2, 12, 6, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DRCO, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 5, 0, MS_SQEEK, MZ_TINY, MR_POISON, 0, M1_ANIMAL | M1_NOHANDS | M1_POIS | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("wererat", S_RODENT, 2, 12, 6, 10, -7, (G_NOGEN | G_NOCORPSE), Attack(AT_BITE, AD_WERE, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 40, 30, 0, MS_SQEEK, MZ_TINY, MR_POISON, 0, M1_NOHANDS | M1_POIS | M1_REGEN | M1_CARNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("rock mole", S_RODENT, 3, 3, 0, 20, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_TUNNEL | M1_ANIMAL | M1_NOHANDS | M1_METALLIVORE, M2_HOSTILE | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("woodchuck", S_RODENT, 3, 3, 0, 20, 0, (G_NOGEN | G_GENO), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_TUNNEL | M1_ANIMAL | M1_NOHANDS | M1_SWIM | M1_HERBIVORE, M2_WANDER | M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("cave spider", S_SPIDER, 1, 12, 3, 0, 0, (G_GENO | G_SGROUP | 2), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 50, 0, MS_SILENT, MZ_TINY, MR_POISON, MR_POISON, M1_CONCEAL | M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE, 0, WHITE));
	_monsters.push_back(new Monster("centipede", S_SPIDER, 2, 4, 3, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DRST, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 50, 0, MS_SILENT, MZ_TINY, MR_POISON, MR_POISON, M1_CONCEAL | M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("giant spider", S_SPIDER, 5, 15, 4, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DRST, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 100, 100, 0, MS_SILENT, MZ_LARGE, MR_POISON, MR_POISON, M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_POIS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, 0, MAGENTA));
	_monsters.push_back(new Monster("scorpion", S_SPIDER, 5, 15, 3, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 1, 2), Attack(AT_CLAW, AD_PHYS, 1, 2), Attack(AT_STNG, AD_DRST, 1, 4), Attack(), Attack(), Attack(), 50, 100, 0, MS_SILENT, MZ_SMALL, MR_POISON, MR_POISON, M1_CONCEAL | M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_POIS | M1_CARNIVORE, M2_HOSTILE, 0, RED));
	_monsters.push_back(new Monster("lurker above", S_TRAPPER, 10, 3, 3, 0, 0, (G_GENO | 2), Attack(AT_ENGL, AD_DGST, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_SILENT, MZ_HUGE, 0, 0, M1_HIDE | M1_FLY | M1_ANIMAL | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_CARNIVORE, M2_HOSTILE | M2_STALK | M2_STRONG, 0, WHITE));
	_monsters.push_back(new Monster("trapper", S_TRAPPER, 12, 3, 3, 0, 0, (G_GENO | 2), Attack(AT_ENGL, AD_DGST, 1, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_SILENT, MZ_HUGE, 0, 0, M1_HIDE | M1_ANIMAL | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_CARNIVORE, M2_HOSTILE | M2_STALK | M2_STRONG, 0, GREEN));
	_monsters.push_back(new Monster("white unicorn", S_UNICORN, 4, 24, 2, 70, 7, (G_GENO | 2), Attack(AT_BUTT, AD_PHYS, 1, 12), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1300, 300, 0, MS_NEIGH, MZ_LARGE, MR_POISON, MR_POISON, M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_JEWELS, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("gray unicorn", S_UNICORN, 4, 24, 2, 70, 0, (G_GENO | 1), Attack(AT_BUTT, AD_PHYS, 1, 12), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1300, 300, 0, MS_NEIGH, MZ_LARGE, MR_POISON, MR_POISON, M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_JEWELS, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("black unicorn", S_UNICORN, 4, 24, 2, 70, -7, (G_GENO | 1), Attack(AT_BUTT, AD_PHYS, 1, 12), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1300, 300, 0, MS_NEIGH, MZ_LARGE, MR_POISON, MR_POISON, M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_JEWELS, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("pony", S_UNICORN, 3, 16, 6, 0, 0, (G_GENO | 2), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), 1300, 250, 0, MS_NEIGH, MZ_MEDIUM, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_DOMESTIC, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("horse", S_UNICORN, 5, 20, 5, 0, 0, (G_GENO | 2), Attack(AT_KICK, AD_PHYS, 1, 8), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), 1500, 300, 0, MS_NEIGH, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_DOMESTIC, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("warhorse", S_UNICORN, 7, 24, 4, 0, 0, (G_GENO | 2), Attack(AT_KICK, AD_PHYS, 1, 10), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), 1800, 350, 0, MS_NEIGH, MZ_LARGE, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_HERBIVORE, M2_WANDER | M2_STRONG | M2_DOMESTIC, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("fog cloud", S_VORTEX, 3, 1, 0, 0, 0, (G_GENO | G_NOCORPSE | 2), Attack(AT_ENGL, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_AMORPHOUS | M1_UNSOLID, M2_HOSTILE | M2_NEUTER, 0, WHITE));
	_monsters.push_back(new Monster("dust vortex", S_VORTEX, 4, 20, 2, 30, 0, (G_GENO | G_NOCORPSE | 2), Attack(AT_ENGL, AD_BLND, 2, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("ice vortex", S_VORTEX, 5, 20, 2, 30, 0, (G_NOHELL | G_GENO | G_NOCORPSE | 1), Attack(AT_ENGL, AD_COLD, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_COLD | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("energy vortex", S_VORTEX, 6, 20, 2, 30, 0, (G_GENO | G_NOCORPSE | 1), Attack(AT_ENGL, AD_ELEC, 1, 6), Attack(AT_ENGL, AD_DREN, 0, 0), Attack(AT_NONE, AD_ELEC, 0, 4), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_ELEC | MR_SLEEP | MR_DISINT | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID, M2_HOSTILE | M2_NEUTER, 0, BOLD_BLUE));
	_monsters.push_back(new Monster("steam vortex", S_VORTEX, 7, 22, 2, 30, 0, (G_HELL | G_GENO | G_NOCORPSE | 2), Attack(AT_ENGL, AD_FIRE, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_FIRE | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("fire vortex", S_VORTEX, 8, 22, 2, 30, 0, (G_HELL | G_GENO | G_NOCORPSE | 1), Attack(AT_ENGL, AD_FIRE, 1, 10), Attack(AT_NONE, AD_FIRE, 0, 4), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_FIRE | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("baby long worm", S_WORM, 8, 3, 5, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 600, 250, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("baby purple worm", S_WORM, 8, 3, 5, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 600, 250, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE, M2_HOSTILE, 0, MAGENTA));
	_monsters.push_back(new Monster("long worm", S_WORM, 8, 3, 5, 10, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_SILENT, MZ_GIGANTIC, 0, 0, M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_OVIPAROUS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE | M2_STRONG | M2_NASTY, 0, YELLOW));
	_monsters.push_back(new Monster("purple worm", S_WORM, 15, 9, 6, 20, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 2, 8), Attack(AT_ENGL, AD_DGST, 1, 10), Attack(), Attack(), Attack(), Attack(), 2700, 700, 0, MS_SILENT, MZ_GIGANTIC, 0, 0, M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY, 0, MAGENTA));
	_monsters.push_back(new Monster("grid bug", S_XAN, 0, 12, 9, 0, 0, (G_GENO | G_SGROUP | G_NOCORPSE | 3), Attack(AT_BITE, AD_ELEC, 1, 1), Attack(), Attack(), Attack(), Attack(), Attack(), 15, 10, 0, MS_BUZZ, MZ_TINY, MR_ELEC | MR_POISON, 0, M1_ANIMAL, M2_HOSTILE, M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("xan", S_XAN, 7, 18, -4, 0, 0, (G_GENO | 3), Attack(AT_STNG, AD_LEGS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 300, 300, 0, MS_BUZZ, MZ_TINY, MR_POISON, MR_POISON, M1_FLY | M1_ANIMAL | M1_NOHANDS | M1_POIS, M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("yellow light", S_LIGHT, 3, 15, 0, 0, 0, (G_NOCORPSE | G_GENO | 4), Attack(AT_EXPL, AD_BLND, 10, 20), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_SMALL, MR_FIRE | MR_COLD | MR_ELEC | MR_DISINT | MR_SLEEP | MR_POISON | MR_ACID | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("black light", S_LIGHT, 5, 15, 0, 0, 0, (G_NOCORPSE | G_GENO | 2), Attack(AT_EXPL, AD_HALU, 10, 12), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_SMALL, MR_FIRE | MR_COLD | MR_ELEC | MR_DISINT | MR_SLEEP | MR_POISON | MR_ACID | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID | M1_SEE_INVIS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, BLACK));
	_monsters.push_back(new Monster("zruty", S_ZRUTY, 9, 8, 3, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(), Attack(), Attack(), 1200, 600, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("couatl", S_ANGEL, 8, 10, 5, 30, 7, (G_NOHELL | G_SGROUP | G_NOCORPSE | 1), Attack(AT_BITE, AD_DRST, 2, 4), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(AT_HUGS, AD_WRAP, 2, 4), Attack(), Attack(), Attack(), 900, 400, 0, MS_HISS, MZ_LARGE, MR_NO_ELBERETH | MR_POISON, 0, M1_FLY | M1_POIS, M2_MINION | M2_STALK | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("Aleax", S_ANGEL, 10, 8, 0, 30, 7, (G_NOHELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_KICK, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), 1450, 400, 0, MS_IMITATE, MZ_HUMAN, MR_NO_ELBERETH | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_HUMANOID | M1_SEE_INVIS, M2_MINION | M2_STALK | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_YELLOW));
	_monsters.push_back(new Monster("Angel", S_ANGEL, 14, 10, -4, 55, 12, (G_NOHELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_MAGC, AD_MAGM, 2, 6), Attack(), Attack(), 1450, 400, 0, MS_CUSS, MZ_HUMAN, MR_NO_ELBERETH | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_FLY | M1_HUMANOID | M1_SEE_INVIS, M2_NOPOLY | M2_MINION | M2_STALK | M2_STRONG | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("ki-rin", S_ANGEL, 16, 18, -5, 90, 15, (G_NOHELL | G_NOCORPSE | 1), Attack(AT_KICK, AD_PHYS, 2, 4), Attack(AT_KICK, AD_PHYS, 2, 4), Attack(AT_BUTT, AD_PHYS, 3, 6), Attack(AT_MAGC, AD_SPEL, 2, 6), Attack(), Attack(), 1450, 400, 0, MS_NEIGH, MZ_LARGE, MR_NO_ELBERETH, 0, M1_FLY | M1_SEE_INVIS, M2_NOPOLY | M2_MINION | M2_STALK | M2_STRONG | M2_NASTY | M2_LORD, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_YELLOW));
	_monsters.push_back(new Monster("Archon", S_ANGEL, 19, 16, -6, 80, 15, (G_NOHELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_GAZE, AD_BLND, 2, 6), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_MAGC, AD_SPEL, 4, 6), Attack(), 1450, 400, 0, MS_CUSS, MZ_LARGE, MR_NO_ELBERETH | MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_FLY | M1_HUMANOID | M1_SEE_INVIS | M1_REGEN, M2_NOPOLY | M2_MINION | M2_STALK | M2_STRONG | M2_NASTY | M2_LORD | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("bat", S_BAT, 0, 22, 8, 0, 0, (G_GENO | G_SGROUP | 1), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 20, 20, 0, MS_SQEEK, MZ_TINY, 0, 0, M1_FLY | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_WANDER, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("giant bat", S_BAT, 2, 22, 7, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_SQEEK, MZ_SMALL, 0, 0, M1_FLY | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_WANDER | M2_HOSTILE, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("raven", S_BAT, 4, 20, 6, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_BLND, 1, 6), Attack(), Attack(), Attack(), Attack(), 40, 20, 0, MS_SQAWK, MZ_SMALL, 0, 0, M1_FLY | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_WANDER | M2_HOSTILE, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("vampire bat", S_BAT, 5, 20, 6, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(AT_BITE, AD_DRST, 0, 0), Attack(), Attack(), Attack(), Attack(), 30, 20, 0, MS_SQEEK, MZ_SMALL, MR_SLEEP | MR_POISON, 0, M1_FLY | M1_ANIMAL | M1_NOHANDS | M1_POIS | M1_REGEN | M1_OMNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("plains centaur", S_CENTAUR, 4, 18, 4, 0, 0, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 2500, 500, 0, MS_HUMANOID, MZ_LARGE, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_STRONG | M2_GREEDY | M2_COLLECT, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("forest centaur", S_CENTAUR, 5, 18, 3, 10, -1, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 2550, 600, 0, MS_HUMANOID, MZ_LARGE, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_STRONG | M2_GREEDY | M2_COLLECT, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("mountain centaur", S_CENTAUR, 6, 20, 2, 10, -3, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 10), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(AT_KICK, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), 2550, 500, 0, MS_HUMANOID, MZ_LARGE, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_STRONG | M2_GREEDY | M2_COLLECT, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("baby gray dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, 0, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, WHITE));
	_monsters.push_back(new Monster("baby silver dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, 0, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BOLD_CYAN));
	_monsters.push_back(new Monster("baby red dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_FIRE, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("baby white dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_COLD, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BOLD_WHITE));
	_monsters.push_back(new Monster("baby orange dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_SLEEP, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BOLD_RED));
	_monsters.push_back(new Monster("baby black dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_DISINT, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BLACK));
	_monsters.push_back(new Monster("baby blue dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_ELEC, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BLUE));
	_monsters.push_back(new Monster("baby green dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_POISON, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE | M1_POIS, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, GREEN));
	_monsters.push_back(new Monster("baby yellow dragon", S_DRAGON, 12, 9, 2, 10, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_ROAR, MZ_HUGE, MR_ACID | MR_STONE, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE | M1_ACID, M2_HOSTILE | M2_STRONG | M2_GREEDY | M2_JEWELS, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("gray dragon", S_DRAGON, 15, 9, -1, 20, 4, (G_GENO | 1), Attack(AT_BREA, AD_MAGM, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, 0, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, WHITE));
	_monsters.push_back(new Monster("silver dragon", S_DRAGON, 15, 9, -1, 20, 4, (G_GENO | 1), Attack(AT_BREA, AD_COLD, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_COLD, 0, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BOLD_CYAN));
	_monsters.push_back(new Monster("red dragon", S_DRAGON, 15, 9, -1, 20, -4, (G_GENO | 1), Attack(AT_BREA, AD_FIRE, 6, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_FIRE, MR_FIRE, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("white dragon", S_DRAGON, 15, 9, -1, 20, -5, (G_GENO | 1), Attack(AT_BREA, AD_COLD, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_COLD, MR_COLD, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BOLD_WHITE));
	_monsters.push_back(new Monster("orange dragon", S_DRAGON, 15, 9, -1, 20, 5, (G_GENO | 1), Attack(AT_BREA, AD_SLEE, 4, 25), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_SLEEP, MR_SLEEP, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BOLD_RED));
	_monsters.push_back(new Monster("black dragon", S_DRAGON, 15, 9, -1, 20, -6, (G_GENO | 1), Attack(AT_BREA, AD_DISN, 4, 10), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_DISINT, MR_DISINT, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BLACK));
	_monsters.push_back(new Monster("blue dragon", S_DRAGON, 15, 9, -1, 20, -7, (G_GENO | 1), Attack(AT_BREA, AD_ELEC, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_ELEC, MR_ELEC, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BLUE));
	_monsters.push_back(new Monster("green dragon", S_DRAGON, 15, 9, -1, 20, 6, (G_GENO | 1), Attack(AT_BREA, AD_DRST, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_POISON, MR_POISON, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE | M1_POIS, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, GREEN));
	_monsters.push_back(new Monster("yellow dragon", S_DRAGON, 15, 9, -1, 20, 7, (G_GENO | 1), Attack(AT_BREA, AD_ACID, 4, 6), Attack(AT_BITE, AD_PHYS, 3, 8), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), 4500, 1500, 0, MS_ROAR, MZ_GIGANTIC, MR_ACID | MR_STONE, MR_STONE, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_SEE_INVIS | M1_OVIPAROUS | M1_CARNIVORE | M1_ACID, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("stalker", S_ELEMENTAL, 8, 12, 3, 0, 0, (G_GENO | 3), Attack(AT_CLAW, AD_PHYS, 4, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 900, 400, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_ANIMAL | M1_FLY | M1_SEE_INVIS, M2_WANDER | M2_STALK | M2_HOSTILE | M2_STRONG, M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("air elemental", S_ELEMENTAL, 8, 36, 2, 30, 0, (G_NOCORPSE | 1), Attack(AT_ENGL, AD_PHYS, 1, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_POISON | MR_STONE, 0, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID | M1_FLY, M2_STRONG | M2_NEUTER, 0, CYAN));
	_monsters.push_back(new Monster("fire elemental", S_ELEMENTAL, 8, 12, 2, 30, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_FIRE, 3, 6), Attack(AT_NONE, AD_FIRE, 0, 4), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUGE, MR_FIRE | MR_POISON | MR_STONE, 0, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_UNSOLID | M1_FLY | M1_NOTAKE, M2_STRONG | M2_NEUTER, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("earth elemental", S_ELEMENTAL, 8, 6, 2, 30, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 4, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 2500, 0, 0, MS_SILENT, MZ_HUGE, MR_FIRE | MR_COLD | MR_POISON | MR_STONE, 0, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_BREATHLESS | M1_WALLWALK | M1_THICK_HIDE, M2_STRONG | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("water elemental", S_ELEMENTAL, 8, 6, 2, 30, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 5, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 2500, 0, 0, MS_SILENT, MZ_HUGE, MR_POISON | MR_STONE, 0, M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_AMPHIBIOUS | M1_SWIM, M2_STRONG | M2_NEUTER, 0, BLUE));
	_monsters.push_back(new Monster("lichen", S_FUNGUS, 0, 1, 9, 0, 0, (G_GENO | 4), Attack(AT_TUCH, AD_STCK, 0, 0), Attack(), Attack(), Attack(), Attack(), Attack(), 20, 200, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, BOLD_GREEN));
	_monsters.push_back(new Monster("brown mold", S_FUNGUS, 1, 0, 9, 0, 0, (G_GENO | 1), Attack(AT_NONE, AD_COLD, 0, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 30, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_POISON, MR_COLD | MR_POISON, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("yellow mold", S_FUNGUS, 1, 0, 9, 0, 0, (G_GENO | 2), Attack(AT_NONE, AD_STUN, 0, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 30, 0, MS_SILENT, MZ_SMALL, MR_POISON, MR_POISON, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_POIS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("green mold", S_FUNGUS, 1, 0, 9, 0, 0, (G_GENO | 1), Attack(AT_NONE, AD_ACID, 0, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 30, 0, MS_SILENT, MZ_SMALL, MR_ACID | MR_STONE, MR_STONE, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_ACID | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, GREEN));
	_monsters.push_back(new Monster("red mold", S_FUNGUS, 1, 0, 9, 0, 0, (G_GENO | 1), Attack(AT_NONE, AD_FIRE, 0, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 30, 0, MS_SILENT, MZ_SMALL, MR_FIRE | MR_POISON, MR_FIRE | MR_POISON, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("shrieker", S_FUNGUS, 3, 1, 7, 0, 0, (G_GENO | 1), Attack(), Attack(), Attack(), Attack(), Attack(), Attack(), 100, 100, 0, MS_SHRIEK, MZ_SMALL, MR_POISON, MR_POISON, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, MAGENTA));
	_monsters.push_back(new Monster("violet fungus", S_FUNGUS, 3, 1, 7, 0, 0, (G_GENO | 2), Attack(AT_TUCH, AD_PHYS, 1, 4), Attack(AT_TUCH, AD_STCK, 0, 0), Attack(), Attack(), Attack(), Attack(), 100, 100, 0, MS_SILENT, MZ_SMALL, MR_POISON, MR_POISON, M1_BREATHLESS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_NOTAKE, M2_HOSTILE | M2_NEUTER, 0, MAGENTA));
	_monsters.push_back(new Monster("gnome", S_GNOME, 1, 6, 10, 4, 0, (G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 650, 100, 0, MS_ORC, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_GNOME | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("gnome lord", S_GNOME, 3, 8, 10, 4, 0, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 700, 120, 0, MS_ORC, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_GNOME | M2_LORD | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("gnomish wizard", S_GNOME, 3, 10, 4, 10, 0, (G_GENO | 1), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), Attack(), 700, 120, 0, MS_ORC, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_GNOME | M2_MAGIC, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_BLUE));
	_monsters.push_back(new Monster("gnome king", S_GNOME, 5, 10, 10, 20, 0, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 750, 150, 0, MS_ORC, MZ_SMALL, 0, 0, M1_HUMANOID | M1_OMNIVORE, M2_GNOME | M2_PRINCE | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("giant", S_GIANT, 6, 6, 0, 0, 2, (G_GENO | G_NOGEN | 1), Attack(AT_WEAP, AD_PHYS, 2, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 2250, 750, 0, MS_BOAST, MZ_HUGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("stone giant", S_GIANT, 6, 6, 0, 0, 2, (G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 2, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 2250, 750, 0, MS_BOAST, MZ_HUGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("hill giant", S_GIANT, 8, 10, 6, 0, -2, (G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 2200, 700, 0, MS_BOAST, MZ_HUGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, CYAN));
	_monsters.push_back(new Monster("fire giant", S_GIANT, 9, 12, 4, 5, 2, (G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 2, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 2250, 750, 0, MS_BOAST, MZ_HUGE, MR_FIRE, MR_FIRE, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_YELLOW));
	_monsters.push_back(new Monster("frost giant", S_GIANT, 10, 12, 3, 10, -3, (G_NOHELL | G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 2, 12), Attack(), Attack(), Attack(), Attack(), Attack(), 2250, 750, 0, MS_BOAST, MZ_HUGE, MR_COLD, MR_COLD, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("storm giant", S_GIANT, 16, 12, 3, 10, -3, (G_GENO | G_SGROUP | 1), Attack(AT_WEAP, AD_PHYS, 2, 12), Attack(), Attack(), Attack(), Attack(), Attack(), 2250, 750, 0, MS_BOAST, MZ_HUGE, MR_ELEC, MR_ELEC, M1_HUMANOID | M1_CARNIVORE, M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_JEWELS, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("ettin", S_GIANT, 10, 12, 3, 0, 0, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(AT_WEAP, AD_PHYS, 3, 6), Attack(), Attack(), Attack(), Attack(), 1700, 500, 0, MS_GRUNT, MZ_HUGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("titan", S_GIANT, 16, 18, -3, 70, 9, (1), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 2300, 900, 0, MS_SPELL, MZ_HUGE, 0, 0, M1_FLY | M1_HUMANOID | M1_OMNIVORE, M2_STRONG | M2_ROCKTHROW | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("minotaur", S_GIANT, 15, 15, 6, 0, 0, (G_GENO | G_NOGEN), Attack(AT_CLAW, AD_PHYS, 3, 10), Attack(AT_CLAW, AD_PHYS, 3, 10), Attack(AT_BUTT, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), 1500, 700, 0, MS_SILENT, MZ_LARGE, MR_NO_ELBERETH, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("jabberwock", S_JABBERWOCK, 15, 12, -2, 50, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 10), Attack(AT_BITE, AD_PHYS, 2, 10), Attack(AT_CLAW, AD_PHYS, 2, 10), Attack(AT_CLAW, AD_PHYS, 2, 10), Attack(), Attack(), 1300, 600, 0, MS_BURBLE, MZ_LARGE, 0, 0, M1_ANIMAL | M1_FLY | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE, BOLD_RED));
	_monsters.push_back(new Monster("lich", S_LICH, 11, 6, 0, 30, -9, (G_GENO | G_NOCORPSE | 1), Attack(AT_TUCH, AD_COLD, 1, 10), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1200, 100, 0, MS_MUMBLE, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, MR_COLD, M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_HOSTILE | M2_MAGIC, M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("demilich", S_LICH, 14, 9, -2, 60, -12, (G_GENO | G_NOCORPSE | 1), Attack(AT_TUCH, AD_COLD, 3, 4), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1200, 100, 0, MS_MUMBLE, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, MR_COLD, M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_HOSTILE | M2_MAGIC, M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("master lich", S_LICH, 17, 9, -4, 90, -15, (G_HELL | G_GENO | G_NOCORPSE | 1), Attack(AT_TUCH, AD_COLD, 3, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1200, 100, 0, MS_MUMBLE, MZ_HUMAN, MR_FIRE | MR_COLD | MR_SLEEP | MR_POISON, MR_FIRE | MR_COLD, M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_HOSTILE | M2_MAGIC, M3_WANTSBOOK | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("arch-lich", S_LICH, 25, 9, -6, 90, -15, (G_HELL | G_GENO | G_NOCORPSE | 1), Attack(AT_TUCH, AD_COLD, 5, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1200, 100, 0, MS_MUMBLE, MZ_HUMAN, MR_FIRE | MR_COLD | MR_SLEEP | MR_ELEC | MR_POISON, MR_FIRE | MR_COLD, M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_HOSTILE | M2_MAGIC, M3_WANTSBOOK | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("kobold mummy", S_MUMMY, 3, 8, 6, 20, -2, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 50, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE, M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("gnome mummy", S_MUMMY, 4, 10, 6, 20, -3, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 650, 50, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_GNOME, M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("orc mummy", S_MUMMY, 5, 10, 5, 20, -4, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 850, 75, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_ORC | M2_GREEDY | M2_JEWELS, M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("dwarf mummy", S_MUMMY, 5, 10, 5, 20, -4, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 900, 150, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_DWARF | M2_GREEDY | M2_JEWELS, M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("elf mummy", S_MUMMY, 6, 12, 4, 30, -5, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 175, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_ELF, M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("human mummy", S_MUMMY, 6, 12, 4, 30, -5, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), 1450, 200, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE, M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("ettin mummy", S_MUMMY, 7, 12, 4, 30, -6, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), 1700, 250, 0, MS_SILENT, MZ_HUGE, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_STRONG, M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("giant mummy", S_MUMMY, 8, 14, 3, 30, -7, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), Attack(), 2050, 375, 0, MS_SILENT, MZ_HUGE, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_HOSTILE | M2_GIANT | M2_STRONG | M2_JEWELS, M3_INFRAVISION, CYAN));
	_monsters.push_back(new Monster("red naga hatchling", S_NAGA, 3, 10, 6, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 100, 0, MS_MUMBLE, MZ_LARGE, MR_FIRE | MR_POISON, MR_FIRE | MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("black naga hatchling", S_NAGA, 3, 10, 6, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 100, 0, MS_MUMBLE, MZ_LARGE, MR_POISON | MR_ACID | MR_STONE, MR_POISON | MR_STONE, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_ACID | M1_NOTAKE | M1_CARNIVORE, M2_STRONG, 0, BLACK));
	_monsters.push_back(new Monster("golden naga hatchling", S_NAGA, 3, 10, 6, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 100, 0, MS_MUMBLE, MZ_LARGE, MR_POISON, MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("guardian naga hatchling", S_NAGA, 3, 10, 6, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 100, 0, MS_MUMBLE, MZ_LARGE, MR_POISON, MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, 0, GREEN));
	_monsters.push_back(new Monster("red naga", S_NAGA, 6, 12, 4, 0, -4, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(AT_BREA, AD_FIRE, 2, 6), Attack(), Attack(), Attack(), Attack(), 2600, 400, 0, MS_MUMBLE, MZ_HUGE, MR_FIRE | MR_POISON, MR_FIRE | MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_OVIPAROUS | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("black naga", S_NAGA, 8, 14, 2, 10, 4, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(AT_SPIT, AD_ACID, 0, 0), Attack(), Attack(), Attack(), Attack(), 2600, 400, 0, MS_MUMBLE, MZ_HUGE, MR_POISON | MR_ACID | MR_STONE, MR_POISON | MR_STONE, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_OVIPAROUS | M1_ACID | M1_NOTAKE | M1_CARNIVORE, M2_STRONG, 0, BLACK));
	_monsters.push_back(new Monster("golden naga", S_NAGA, 10, 14, 2, 70, 5, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(AT_MAGC, AD_SPEL, 4, 6), Attack(), Attack(), Attack(), Attack(), 2600, 400, 0, MS_MUMBLE, MZ_HUGE, MR_POISON, MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_OVIPAROUS | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("guardian naga", S_NAGA, 12, 16, 0, 50, 7, (G_GENO | 1), Attack(AT_BITE, AD_PLYS, 1, 6), Attack(AT_SPIT, AD_DRST, 1, 6), Attack(AT_HUGS, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), 2600, 400, 0, MS_MUMBLE, MZ_HUGE, MR_POISON, MR_POISON, M1_NOLIMBS | M1_SLITHY | M1_THICK_HIDE | M1_OVIPAROUS | M1_POIS | M1_NOTAKE | M1_OMNIVORE, M2_STRONG, 0, GREEN));
	_monsters.push_back(new Monster("ogre", S_OGRE, 5, 10, 5, 0, -3, (G_SGROUP | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 5), Attack(), Attack(), Attack(), Attack(), Attack(), 1600, 500, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("ogre lord", S_OGRE, 7, 12, 3, 30, -5, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1700, 700, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_STRONG | M2_LORD | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("ogre king", S_OGRE, 9, 14, 4, 60, -7, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 3, 5), Attack(), Attack(), Attack(), Attack(), Attack(), 1700, 750, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_CARNIVORE, M2_STRONG | M2_PRINCE | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("gray ooze", S_PUDDING, 3, 1, 8, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_RUST, 2, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 250, 0, MS_SILENT, MZ_MEDIUM, MR_FIRE | MR_COLD | MR_POISON | MR_ACID | MR_STONE, MR_FIRE | MR_COLD | MR_POISON, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_OMNIVORE | M1_ACID, M2_HOSTILE | M2_NEUTER, 0, WHITE));
	_monsters.push_back(new Monster("brown pudding", S_PUDDING, 5, 3, 8, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DCAY, 0, 0), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 250, 0, MS_SILENT, MZ_MEDIUM, MR_COLD | MR_ELEC | MR_POISON | MR_ACID | MR_STONE, MR_COLD | MR_ELEC | MR_POISON, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_OMNIVORE | M1_ACID, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("black pudding", S_PUDDING, 10, 6, 6, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_CORR, 3, 8), Attack(AT_NONE, AD_CORR, 0, 0), Attack(), Attack(), Attack(), Attack(), 900, 250, 0, MS_SILENT, MZ_LARGE, MR_COLD | MR_ELEC | MR_POISON | MR_ACID | MR_STONE, MR_COLD | MR_ELEC | MR_POISON, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_OMNIVORE | M1_ACID, M2_HOSTILE | M2_NEUTER, 0, BLACK));
	_monsters.push_back(new Monster("green slime", S_PUDDING, 6, 6, 6, 0, 0, (G_HELL | G_GENO | 1), Attack(AT_TUCH, AD_SLIM, 1, 4), Attack(AT_NONE, AD_SLIM, 0, 0), Attack(), Attack(), Attack(), Attack(), 400, 150, 0, MS_SILENT, MZ_LARGE, MR_COLD | MR_ELEC | MR_POISON | MR_ACID | MR_STONE, 0, M1_BREATHLESS | M1_AMORPHOUS | M1_NOEYES | M1_NOLIMBS | M1_NOHEAD | M1_MINDLESS | M1_OMNIVORE | M1_ACID | M1_POIS, M2_HOSTILE | M2_NEUTER, 0, GREEN));
	_monsters.push_back(new Monster("quantum mechanic", S_QUANTMECH, 7, 12, 3, 10, 0, (G_GENO | 3), Attack(AT_CLAW, AD_TLPT, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 20, 0, MS_HUMANOID, MZ_HUMAN, MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE | M1_POIS | M1_TPORT, M2_HOSTILE, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("rust monster", S_RUSTMONST, 5, 18, 2, 0, 0, (G_GENO | 2), Attack(AT_TUCH, AD_RUST, 0, 0), Attack(AT_TUCH, AD_RUST, 0, 0), Attack(AT_NONE, AD_RUST, 0, 0), Attack(), Attack(), Attack(), 1000, 250, 0, MS_SILENT, MZ_MEDIUM, 0, 0, M1_SWIM | M1_ANIMAL | M1_NOHANDS | M1_METALLIVORE, M2_HOSTILE, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("disenchanter", S_RUSTMONST, 12, 12, -10, 0, -3, (G_HELL | G_GENO | 2), Attack(AT_CLAW, AD_ENCH, 4, 4), Attack(AT_NONE, AD_ENCH, 0, 0), Attack(), Attack(), Attack(), Attack(), 750, 200, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_CARNIVORE, M2_HOSTILE, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("garter snake", S_SNAKE, 1, 8, 8, 0, 0, (G_LGROUP | G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 50, 60, 0, MS_HISS, MZ_TINY, 0, 0, M1_SWIM | M1_CONCEAL | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_OVIPAROUS | M1_CARNIVORE | M1_NOTAKE, 0, 0, GREEN));
	_monsters.push_back(new Monster("snake", S_SNAKE, 4, 15, 3, 0, 0, (G_GENO | 2), Attack(AT_BITE, AD_DRST, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 100, 80, 0, MS_HISS, MZ_SMALL, MR_POISON, MR_POISON, M1_SWIM | M1_CONCEAL | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_POIS | M1_OVIPAROUS | M1_CARNIVORE | M1_NOTAKE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("water moccasin", S_SNAKE, 4, 15, 3, 0, 0, (G_GENO | G_NOGEN | G_LGROUP), Attack(AT_BITE, AD_DRST, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 150, 80, 0, MS_HISS, MZ_SMALL, MR_POISON, MR_POISON, M1_SWIM | M1_CONCEAL | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_POIS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, 0, RED));
	_monsters.push_back(new Monster("pit viper", S_SNAKE, 6, 15, 2, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DRST, 1, 4), Attack(AT_BITE, AD_DRST, 1, 4), Attack(), Attack(), Attack(), Attack(), 100, 60, 0, MS_HISS, MZ_MEDIUM, MR_POISON, MR_POISON, M1_SWIM | M1_CONCEAL | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_POIS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("python", S_SNAKE, 6, 3, 5, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(AT_TUCH, AD_PHYS, 0, 0), Attack(AT_HUGS, AD_WRAP, 1, 4), Attack(AT_HUGS, AD_PHYS, 2, 4), Attack(), Attack(), 250, 100, 0, MS_HISS, MZ_LARGE, 0, 0, M1_SWIM | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE | M2_STRONG, M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("cobra", S_SNAKE, 6, 18, 2, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_DRST, 2, 4), Attack(AT_SPIT, AD_BLND, 0, 0), Attack(), Attack(), Attack(), Attack(), 250, 100, 0, MS_HISS, MZ_MEDIUM, MR_POISON, MR_POISON, M1_SWIM | M1_CONCEAL | M1_NOLIMBS | M1_ANIMAL | M1_SLITHY | M1_POIS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, 0, BLUE));
	_monsters.push_back(new Monster("troll", S_TROLL, 7, 12, 4, 0, -3, (G_GENO | 2), Attack(AT_WEAP, AD_PHYS, 4, 2), Attack(AT_CLAW, AD_PHYS, 4, 2), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 800, 350, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_REGEN | M1_CARNIVORE, M2_STRONG | M2_STALK | M2_HOSTILE, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("ice troll", S_TROLL, 9, 10, 2, 20, -3, (G_NOHELL | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_COLD, 2, 6), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 1000, 300, 0, MS_GRUNT, MZ_LARGE, MR_COLD, MR_COLD, M1_HUMANOID | M1_REGEN | M1_CARNIVORE, M2_STRONG | M2_STALK | M2_HOSTILE, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("rock troll", S_TROLL, 9, 12, 0, 0, -3, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 3, 6), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 1200, 300, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_REGEN | M1_CARNIVORE, M2_STRONG | M2_STALK | M2_HOSTILE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, CYAN));
	_monsters.push_back(new Monster("water troll", S_TROLL, 11, 14, 4, 40, -3, (G_NOGEN | G_GENO), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 1200, 350, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_REGEN | M1_CARNIVORE | M1_SWIM, M2_STRONG | M2_STALK | M2_HOSTILE, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("Olog-hai", S_TROLL, 13, 12, -4, 0, -7, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 3, 6), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), 1500, 400, 0, MS_GRUNT, MZ_LARGE, 0, 0, M1_HUMANOID | M1_REGEN | M1_CARNIVORE, M2_STRONG | M2_STALK | M2_HOSTILE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("umber hulk", S_UMBER, 9, 6, 2, 25, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_BITE, AD_PHYS, 2, 5), Attack(AT_GAZE, AD_CONF, 0, 0), Attack(), Attack(), 1200, 500, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_TUNNEL | M1_CARNIVORE, M2_STRONG, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("vampire", S_VAMPIRE, 10, 12, 1, 25, -8, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_BITE, AD_DRLI, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_VAMPIRE, MZ_HUMAN, MR_SLEEP | MR_POISON, 0, M1_FLY | M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("vampire lord", S_VAMPIRE, 12, 14, 0, 50, -9, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_BITE, AD_DRLI, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_VAMPIRE, MZ_HUMAN, MR_SLEEP | MR_POISON, 0, M1_FLY | M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_LORD | M2_MALE, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("Vlad the Impaler", S_VAMPIRE, 14, 18, -3, 80, -10, (G_NOGEN | G_NOCORPSE | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 10), Attack(AT_BITE, AD_DRLI, 1, 10), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_VAMPIRE, MZ_HUMAN, MR_SLEEP | MR_POISON, 0, M1_FLY | M1_BREATHLESS | M1_HUMANOID | M1_POIS | M1_REGEN, M2_NOPOLY | M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY | M2_PRINCE | M2_MALE, M3_WAITFORU | M3_WANTSCAND | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("barrow wight", S_WRAITH, 3, 12, 5, 5, -3, (G_GENO | G_NOCORPSE | 1), Attack(AT_WEAP, AD_DRLI, 0, 0), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), 1200, 0, 0, MS_SPELL, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_HUMANOID, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_COLLECT, 0, WHITE));
	_monsters.push_back(new Monster("wraith", S_WRAITH, 6, 12, 4, 15, -6, (G_GENO | 2), Attack(AT_TUCH, AD_DRLI, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_BREATHLESS | M1_FLY | M1_HUMANOID | M1_UNSOLID, M2_UNDEAD | M2_STALK | M2_HOSTILE, 0, BLACK));
	_monsters.push_back(new Monster("Nazgul", S_WRAITH, 13, 12, 0, 25, -17, (G_GENO | G_NOCORPSE | 1), Attack(AT_WEAP, AD_DRLI, 1, 4), Attack(AT_BREA, AD_SLEE, 2, 25), Attack(), Attack(), Attack(), Attack(), 1450, 0, 0, MS_SPELL, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_HUMANOID, M2_NOPOLY | M2_UNDEAD | M2_STALK | M2_STRONG | M2_HOSTILE | M2_MALE | M2_COLLECT, 0, MAGENTA));
	_monsters.push_back(new Monster("xorn", S_XORN, 8, 9, -2, 20, 0, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 4, 6), Attack(), Attack(), 1200, 700, 0, MS_ROAR, MZ_MEDIUM, MR_FIRE | MR_COLD | MR_STONE, MR_STONE, M1_BREATHLESS | M1_WALLWALK | M1_THICK_HIDE | M1_METALLIVORE, M2_HOSTILE | M2_STRONG, 0, YELLOW));
	_monsters.push_back(new Monster("monkey", S_YETI, 2, 12, 6, 0, 0, (G_GENO | 1), Attack(AT_CLAW, AD_SITM, 0, 0), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), 100, 50, 0, MS_GROWL, MZ_SMALL, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, 0, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("ape", S_YETI, 4, 12, 6, 0, 0, (G_GENO | G_SGROUP | 2), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), 1100, 500, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_STRONG, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("owlbear", S_YETI, 5, 12, 5, 0, 0, (G_GENO | 3), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_HUGS, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), 1700, 700, 0, MS_ROAR, MZ_LARGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("yeti", S_YETI, 5, 15, 6, 0, 0, (G_GENO | 2), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), 1600, 700, 0, MS_GROWL, MZ_LARGE, MR_COLD, MR_COLD, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("carnivorous ape", S_YETI, 6, 12, 6, 0, 0, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_HUGS, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), 1250, 550, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_CARNIVORE, M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("sasquatch", S_YETI, 7, 15, 6, 0, 2, (G_GENO | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_KICK, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), 1550, 750, 0, MS_GROWL, MZ_LARGE, 0, 0, M1_ANIMAL | M1_HUMANOID | M1_SEE_INVIS | M1_OMNIVORE, M2_STRONG, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("kobold zombie", S_ZOMBIE, 0, 6, 10, 0, -2, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 50, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_STALK | M2_HOSTILE, M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("gnome zombie", S_ZOMBIE, 1, 6, 10, 0, -2, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 5), Attack(), Attack(), Attack(), Attack(), Attack(), 650, 50, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_GNOME, M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("orc zombie", S_ZOMBIE, 2, 6, 9, 0, -3, (G_GENO | G_SGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 850, 75, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_ORC, M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("dwarf zombie", S_ZOMBIE, 2, 6, 9, 0, -3, (G_GENO | G_SGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 900, 150, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_DWARF, M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("elf zombie", S_ZOMBIE, 3, 6, 9, 0, -3, (G_GENO | G_SGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 7), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 175, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_ELF, M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("human zombie", S_ZOMBIE, 4, 6, 8, 0, -3, (G_GENO | G_SGROUP | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 200, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_UNDEAD | M2_STALK | M2_HOSTILE, M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("ettin zombie", S_ZOMBIE, 6, 8, 6, 0, -4, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 10), Attack(AT_CLAW, AD_PHYS, 1, 10), Attack(), Attack(), Attack(), Attack(), 1700, 250, 0, MS_SILENT, MZ_HUGE, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_STRONG, M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("giant zombie", S_ZOMBIE, 8, 8, 6, 0, -4, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), 2050, 375, 0, MS_SILENT, MZ_HUGE, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_UNDEAD | M2_STALK | M2_HOSTILE | M2_GIANT | M2_STRONG, M3_INFRAVISION, CYAN));
	_monsters.push_back(new Monster("ghoul", S_ZOMBIE, 3, 6, 10, 0, -2, (G_GENO | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PLYS, 1, 2), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), 400, 50, 0, MS_SILENT, MZ_SMALL, MR_COLD | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_POIS, M2_UNDEAD | M2_WANDER | M2_HOSTILE, M3_INFRAVISION, BLACK));
	_monsters.push_back(new Monster("skeleton", S_ZOMBIE, 12, 8, 4, 0, 0, (G_NOCORPSE | G_NOGEN), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_TUCH, AD_SLOW, 1, 6), Attack(), Attack(), Attack(), Attack(), 300, 5, 0, MS_BONES, MZ_HUMAN, MR_COLD | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_UNDEAD | M2_WANDER | M2_HOSTILE | M2_STRONG | M2_COLLECT | M2_NASTY, M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("straw golem", S_GOLEM, 3, 12, 10, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 2), Attack(AT_CLAW, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), 400, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_HOSTILE | M2_NEUTER, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("paper golem", S_GOLEM, 3, 12, 10, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 400, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_HOSTILE | M2_NEUTER, 0, BOLD_WHITE));
	_monsters.push_back(new Monster("rope golem", S_GOLEM, 4, 9, 8, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_HUGS, AD_PHYS, 6, 1), Attack(), Attack(), Attack(), 450, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("gold golem", S_GOLEM, 5, 9, 6, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 3), Attack(AT_CLAW, AD_PHYS, 2, 3), Attack(), Attack(), Attack(), Attack(), 450, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON | MR_ACID, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_HOSTILE | M2_NEUTER, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("leather golem", S_GOLEM, 6, 6, 6, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 800, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("wood golem", S_GOLEM, 7, 3, 4, 0, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 900, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_HOSTILE | M2_NEUTER, 0, YELLOW));
	_monsters.push_back(new Monster("flesh golem", S_GOLEM, 9, 8, 9, 30, 0, (1), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), 1400, 600, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID, M2_HOSTILE | M2_STRONG, 0, RED));
	_monsters.push_back(new Monster("clay golem", S_GOLEM, 11, 7, 7, 40, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 3, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 1550, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_HOSTILE | M2_STRONG, 0, YELLOW));
	_monsters.push_back(new Monster("stone golem", S_GOLEM, 14, 6, 5, 50, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 3, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1900, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON | MR_STONE, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_HOSTILE | M2_STRONG, 0, WHITE));
	_monsters.push_back(new Monster("glass golem", S_GOLEM, 16, 6, 1, 50, 0, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(AT_CLAW, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), 1800, 0, 0, MS_SILENT, MZ_LARGE, MR_SLEEP | MR_POISON | MR_ACID, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE, M2_HOSTILE | M2_STRONG, 0, CYAN));
	_monsters.push_back(new Monster("iron golem", S_GOLEM, 18, 6, 3, 60, 0, (G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(AT_BREA, AD_DRST, 4, 6), Attack(), Attack(), Attack(), Attack(), 2000, 0, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_BREATHLESS | M1_MINDLESS | M1_HUMANOID | M1_THICK_HIDE | M1_POIS, M2_HOSTILE | M2_STRONG | M2_COLLECT, 0, CYAN));
	_monsters.push_back(new Monster("human", S_HUMAN, 0, 12, 10, 0, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("wererat", S_HUMAN, 2, 12, 10, 10, -7, (1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_WERE, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_REGEN | M1_OMNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE | M2_HUMAN | M2_COLLECT, M3_INFRAVISIBLE, YELLOW));
	_monsters.push_back(new Monster("werejackal", S_HUMAN, 2, 12, 10, 10, -7, (1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_WERE, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_REGEN | M1_OMNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE | M2_HUMAN | M2_COLLECT, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("werewolf", S_HUMAN, 5, 12, 10, 20, -7, (1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_WERE, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_REGEN | M1_OMNIVORE, M2_NOPOLY | M2_WERE | M2_HOSTILE | M2_HUMAN | M2_COLLECT, M3_INFRAVISIBLE, BOLD_RED));
	_monsters.push_back(new Monster("elf", S_HUMAN, 10, 12, 10, 2, -3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_NOPOLY | M2_ELF | M2_STRONG | M2_COLLECT, M3_INFRAVISION | M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("Woodland-elf", S_HUMAN, 4, 12, 10, 10, -5, (G_GENO | G_SGROUP | 2), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_ELF | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, GREEN));
	_monsters.push_back(new Monster("Green-elf", S_HUMAN, 5, 12, 10, 10, -6, (G_GENO | G_SGROUP | 2), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_ELF | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_GREEN));
	_monsters.push_back(new Monster("Grey-elf", S_HUMAN, 6, 12, 10, 10, -7, (G_GENO | G_SGROUP | 2), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_ELF | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("elf-lord", S_HUMAN, 8, 12, 10, 20, -9, (G_GENO | G_SGROUP | 2), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_ELF | M2_STRONG | M2_LORD | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_BLUE));
	_monsters.push_back(new Monster("Elvenking", S_HUMAN, 9, 12, 10, 25, -10, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), 800, 350, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, MR_SLEEP, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS, M2_ELF | M2_STRONG | M2_PRINCE | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("doppelganger", S_HUMAN, 9, 12, 5, 20, 0, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 12), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_IMITATE, MZ_HUMAN, MR_NO_ELBERETH | MR_SLEEP, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("nurse", S_HUMAN, 11, 6, 0, 0, 0, (G_GENO | 3), Attack(AT_CLAW, AD_HEAL, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_NURSE, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, MR_POISON, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_HOSTILE, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("shopkeeper", S_HUMAN, 12, 18, 0, 50, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 4, 4), Attack(AT_WEAP, AD_PHYS, 4, 4), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SELL, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("guard", S_HUMAN, 12, 12, 10, 40, 10, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARD, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("prisoner", S_HUMAN, 12, 12, 10, 0, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_DJINNI, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE | M3_CLOSE, BOLD_WHITE));
	_monsters.push_back(new Monster("Oracle", S_HUMAN, 12, 0, 0, 50, 0, (G_NOGEN | G_UNIQ), Attack(AT_NONE, AD_MAGM, 0, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_ORACLE, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_FEMALE, M3_INFRAVISIBLE, BOLD_BLUE));
	_monsters.push_back(new Monster("aligned priest", S_HUMAN, 12, 12, 10, 50, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(AT_KICK, AD_PHYS, 1, 4), Attack(AT_MAGC, AD_CLRC, 0, 0), Attack(), Attack(), Attack(), 1450, 400, 0, MS_PRIEST, MZ_HUMAN, MR_NO_ELBERETH | MR_ELEC, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_LORD | M2_PEACEFUL | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("high priest", S_HUMAN, 25, 15, 7, 70, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(AT_KICK, AD_PHYS, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(), Attack(), 1450, 400, 0, MS_PRIEST, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_HUMANOID | M1_SEE_INVIS | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MINION | M2_PRINCE | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("soldier", S_HUMAN, 6, 10, 10, 0, -2, (G_SGROUP | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("sergeant", S_HUMAN, 8, 10, 10, 5, -3, (G_SGROUP | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("lieutenant", S_HUMAN, 10, 10, 10, 15, -4, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 3, 4), Attack(AT_WEAP, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("captain", S_HUMAN, 12, 10, 10, 15, -5, (G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 4, 4), Attack(AT_WEAP, AD_PHYS, 4, 4), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BLUE));
	_monsters.push_back(new Monster("watchman", S_HUMAN, 6, 10, 10, 0, -2, (G_SGROUP | G_NOGEN | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("watch captain", S_HUMAN, 10, 10, 10, 15, -4, (G_NOGEN | G_GENO | 1), Attack(AT_WEAP, AD_PHYS, 3, 4), Attack(AT_WEAP, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SOLDIER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_MERC | M2_STALK | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("Medusa", S_HUMAN, 20, 12, 2, 50, -15, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_GAZE, AD_STON, 0, 0), Attack(AT_BITE, AD_DRST, 1, 6), Attack(), Attack(), 1450, 400, 0, MS_HISS, MZ_LARGE, MR_NO_ELBERETH | MR_POISON | MR_STONE, MR_POISON | MR_STONE, M1_FLY | M1_SWIM | M1_AMPHIBIOUS | M1_HUMANOID | M1_POIS | M1_OMNIVORE, M2_NOPOLY | M2_HOSTILE | M2_STRONG | M2_PNAME | M2_FEMALE, M3_WAITFORU | M3_INFRAVISIBLE, BOLD_GREEN));
	_monsters.push_back(new Monster("Wizard of Yendor", S_HUMAN, 30, 12, -8, 100, A_NONE, (G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_SAMU, 2, 12), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_CUSS, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_POISON, MR_FIRE | MR_POISON, M1_FLY | M1_BREATHLESS | M1_HUMANOID | M1_REGEN | M1_SEE_INVIS | M1_TPORT | M1_TPORT_CNTRL | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_PRINCE | M2_MALE | M2_MAGIC, M3_COVETOUS | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Croesus", S_HUMAN, 20, 15, 0, 40, 15, (G_UNIQ | G_NOGEN), Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARD, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_SEE_INVIS | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_PNAME | M2_PRINCE | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("ghost", S_GHOST, 10, 3, -5, 50, -5, (G_NOCORPSE | G_NOGEN), Attack(AT_TUCH, AD_PHYS, 1, 1), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 0, 0, MS_SILENT, MZ_HUMAN, MR_COLD | MR_DISINT | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_WALLWALK | M1_HUMANOID | M1_UNSOLID, M2_NOPOLY | M2_UNDEAD | M2_STALK | M2_HOSTILE, M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("shade", S_GHOST, 12, 10, 10, 0, 0, (G_NOCORPSE | G_NOGEN), Attack(AT_TUCH, AD_PLYS, 2, 6), Attack(AT_TUCH, AD_SLOW, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 0, 0, MS_WAIL, MZ_HUMAN, MR_COLD | MR_DISINT | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_BREATHLESS | M1_WALLWALK | M1_HUMANOID | M1_UNSOLID | M1_SEE_INVIS, M2_NOPOLY | M2_UNDEAD | M2_WANDER | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISION, BLACK));
	_monsters.push_back(new Monster("water demon", S_DEMON, 8, 12, -4, 30, -7, (G_NOCORPSE | G_NOGEN), Attack(AT_WEAP, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), 1450, 400, 0, MS_DJINNI, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_POIS | M1_SWIM, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, BLUE));
	_monsters.push_back(new Monster("horned devil", S_DEMON, 6, 9, -5, 50, 11, (G_HELL | G_NOCORPSE | 2), Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_BITE, AD_PHYS, 2, 3), Attack(AT_STNG, AD_PHYS, 1, 3), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_POIS | M1_THICK_HIDE, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, YELLOW));
	_monsters.push_back(new Monster("succubus", S_DEMON, 6, 12, 0, 70, -9, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_DRLI, 2, 6), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SEDUCE, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_FLY | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_FEMALE, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("incubus", S_DEMON, 6, 12, 0, 70, -9, (G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_DRLI, 2, 6), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SEDUCE, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_FLY | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_MALE, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("erinys", S_DEMON, 7, 12, 2, 30, 10, (G_HELL | G_NOCORPSE | G_SGROUP | 2), Attack(AT_WEAP, AD_DRST, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("barbed devil", S_DEMON, 8, 12, 0, 35, 8, (G_HELL | G_NOCORPSE | G_SGROUP | 2), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_STNG, AD_PHYS, 3, 4), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_POIS | M1_THICK_HIDE, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("marilith", S_DEMON, 7, 12, -6, 80, -12, (G_HELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), 1450, 400, 0, MS_CUSS, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_SLITHY | M1_SEE_INVIS | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("vrock", S_DEMON, 8, 12, 0, 50, -9, (G_HELL | G_NOCORPSE | G_SGROUP | 2), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("hezrou", S_DEMON, 9, 6, -2, 55, -10, (G_HELL | G_NOCORPSE | G_SGROUP | 2), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_CLAW, AD_PHYS, 1, 3), Attack(AT_BITE, AD_PHYS, 4, 4), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("bone devil", S_DEMON, 9, 15, -1, 40, -9, (G_HELL | G_NOCORPSE | G_SGROUP | 2), Attack(AT_WEAP, AD_PHYS, 3, 4), Attack(AT_STNG, AD_DRST, 2, 4), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("ice devil", S_DEMON, 11, 6, -4, 55, -12, (G_HELL | G_NOCORPSE | 2), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(AT_STNG, AD_COLD, 3, 4), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_COLD | MR_POISON, 0, M1_SEE_INVIS | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, BOLD_WHITE));
	_monsters.push_back(new Monster("nalfeshnee", S_DEMON, 11, 9, -1, 65, -11, (G_HELL | G_NOCORPSE | 1), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_CLAW, AD_PHYS, 1, 4), Attack(AT_BITE, AD_PHYS, 2, 4), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), 1450, 400, 0, MS_SPELL, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_HUMANOID | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("pit fiend", S_DEMON, 13, 6, -3, 65, -13, (G_HELL | G_NOCORPSE | 2), Attack(AT_WEAP, AD_PHYS, 4, 2), Attack(AT_WEAP, AD_PHYS, 4, 2), Attack(AT_HUGS, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GROWL, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_SEE_INVIS | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("balrog", S_DEMON, 16, 5, -2, 75, -14, (G_HELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 8, 4), Attack(AT_WEAP, AD_PHYS, 4, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_DEMON | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, RED));
	_monsters.push_back(new Monster("Juiblex", S_DEMON, 50, 3, -7, 65, -15, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_ENGL, AD_DISE, 4, 10), Attack(AT_SPIT, AD_ACID, 3, 6), Attack(), Attack(), Attack(), Attack(), 1500, 0, 0, MS_GURGLE, MZ_LARGE, MR_FIRE | MR_POISON | MR_ACID | MR_STONE, 0, M1_AMPHIBIOUS | M1_AMORPHOUS | M1_NOHEAD | M1_FLY | M1_SEE_INVIS | M1_ACID | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_LORD | M2_MALE, M3_WAITFORU | M3_WANTSAMUL | M3_INFRAVISION, BOLD_GREEN));
	_monsters.push_back(new Monster("Yeenoghu", S_DEMON, 56, 18, -5, 80, -15, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 3, 6), Attack(AT_WEAP, AD_CONF, 2, 8), Attack(AT_CLAW, AD_PLYS, 1, 6), Attack(AT_MAGC, AD_MAGM, 2, 6), Attack(), Attack(), 900, 500, 0, MS_ORC, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_LORD | M2_MALE | M2_COLLECT, M3_WANTSAMUL | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Orcus", S_DEMON, 66, 9, -6, 85, -20, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 3, 6), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_CLAW, AD_PHYS, 3, 4), Attack(AT_MAGC, AD_SPEL, 8, 6), Attack(AT_STNG, AD_DRST, 2, 4), Attack(), 1500, 500, 0, MS_ORC, MZ_HUGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_PRINCE | M2_MALE | M2_COLLECT, M3_WAITFORU | M3_WANTSBOOK | M3_WANTSAMUL | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Geryon", S_DEMON, 72, 3, -3, 75, 15, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_PHYS, 3, 6), Attack(AT_CLAW, AD_PHYS, 3, 6), Attack(AT_STNG, AD_DRST, 2, 4), Attack(), Attack(), Attack(), 1500, 500, 0, MS_BRIBE, MZ_HUGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS | M1_SLITHY, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_PRINCE | M2_MALE, M3_WANTSAMUL | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Dispater", S_DEMON, 78, 15, -2, 80, 15, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 4, 6), Attack(AT_MAGC, AD_SPEL, 6, 6), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_BRIBE, MZ_HUMAN, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS | M1_HUMANOID, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_PRINCE | M2_MALE | M2_COLLECT, M3_WANTSAMUL | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Baalzebub", S_DEMON, 89, 9, -5, 85, 20, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_BITE, AD_DRST, 2, 6), Attack(AT_GAZE, AD_STUN, 2, 6), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_BRIBE, MZ_LARGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_PRINCE | M2_MALE, M3_WANTSAMUL | M3_WAITFORU | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Asmodeus", S_DEMON, 105, 12, -7, 90, 20, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_PHYS, 4, 4), Attack(AT_MAGC, AD_COLD, 6, 6), Attack(), Attack(), Attack(), Attack(), 1500, 500, 0, MS_BRIBE, MZ_HUGE, MR_FIRE | MR_COLD | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_HUMANOID | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY | M2_PRINCE | M2_MALE, M3_WANTSAMUL | M3_WAITFORU | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Demogorgon", S_DEMON, 106, 15, -8, 95, -20, (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_MAGC, AD_SPEL, 8, 6), Attack(AT_STNG, AD_DRLI, 1, 4), Attack(AT_CLAW, AD_DISE, 1, 6), Attack(AT_CLAW, AD_DISE, 1, 6), Attack(), Attack(), 1500, 500, 0, MS_GROWL, MZ_HUGE, MR_FIRE | MR_POISON, 0, M1_FLY | M1_SEE_INVIS | M1_NOHANDS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_PRINCE | M2_MALE, M3_WANTSAMUL | M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Death", S_DEMON, 30, 12, -5, 100, 0, (G_UNIQ | G_NOGEN), Attack(AT_TUCH, AD_DETH, 8, 8), Attack(AT_TUCH, AD_DETH, 8, 8), Attack(), Attack(), Attack(), Attack(), 1450, 1, 0, MS_RIDER, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_HUMANOID | M1_REGEN | M1_SEE_INVIS | M1_TPORT_CNTRL, M2_NOPOLY | M2_STALK | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Pestilence", S_DEMON, 30, 12, -5, 100, 0, (G_UNIQ | G_NOGEN), Attack(AT_TUCH, AD_PEST, 8, 8), Attack(AT_TUCH, AD_PEST, 8, 8), Attack(), Attack(), Attack(), Attack(), 1450, 1, 0, MS_RIDER, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_HUMANOID | M1_REGEN | M1_SEE_INVIS | M1_TPORT_CNTRL, M2_NOPOLY | M2_STALK | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("Famine", S_DEMON, 30, 12, -5, 100, 0, (G_UNIQ | G_NOGEN), Attack(AT_TUCH, AD_FAMN, 8, 8), Attack(AT_TUCH, AD_FAMN, 8, 8), Attack(), Attack(), Attack(), Attack(), 1450, 1, 0, MS_RIDER, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_COLD | MR_ELEC | MR_SLEEP | MR_POISON | MR_STONE, 0, M1_FLY | M1_HUMANOID | M1_REGEN | M1_SEE_INVIS | M1_TPORT_CNTRL, M2_NOPOLY | M2_STALK | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY, M3_INFRAVISIBLE | M3_INFRAVISION, MAGENTA));
	_monsters.push_back(new Monster("djinni", S_DEMON, 7, 12, 4, 30, 0, (G_NOGEN | G_NOCORPSE), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(), Attack(), Attack(), Attack(), Attack(), 1500, 400, 0, MS_DJINNI, MZ_HUMAN, MR_POISON | MR_STONE, 0, M1_HUMANOID | M1_FLY | M1_POIS, M2_NOPOLY | M2_STALK | M2_COLLECT, M3_INFRAVISIBLE, BOLD_YELLOW));
	_monsters.push_back(new Monster("sandestin", S_DEMON, 13, 12, 4, 60, -5, (G_HELL | G_NOCORPSE | 1), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), 1500, 400, 0, MS_CUSS, MZ_HUMAN, MR_STONE, 0, M1_HUMANOID, M2_NOPOLY | M2_STALK | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE | M3_INFRAVISION, WHITE));
	_monsters.push_back(new Monster("jellyfish", S_EEL, 3, 3, 6, 0, 0, (G_GENO | G_NOGEN), Attack(AT_STNG, AD_DRST, 3, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 80, 20, 0, MS_SILENT, MZ_SMALL, MR_POISON, MR_POISON, M1_SWIM | M1_AMPHIBIOUS | M1_SLITHY | M1_NOLIMBS | M1_NOTAKE | M1_POIS, M2_HOSTILE, 0, BLUE));
	_monsters.push_back(new Monster("piranha", S_EEL, 5, 12, 4, 0, 0, (G_GENO | G_NOGEN | G_SGROUP), Attack(AT_BITE, AD_PHYS, 2, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 60, 30, 0, MS_SILENT, MZ_SMALL, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, 0, RED));
	_monsters.push_back(new Monster("shark", S_EEL, 7, 12, 2, 0, 0, (G_GENO | G_NOGEN), Attack(AT_BITE, AD_PHYS, 5, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 500, 350, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE | M1_OVIPAROUS | M1_THICK_HIDE | M1_NOTAKE, M2_HOSTILE, 0, WHITE));
	_monsters.push_back(new Monster("giant eel", S_EEL, 5, 9, -1, 0, 0, (G_GENO | G_NOGEN), Attack(AT_BITE, AD_PHYS, 3, 6), Attack(AT_TUCH, AD_WRAP, 0, 0), Attack(), Attack(), Attack(), Attack(), 200, 250, 0, MS_SILENT, MZ_HUGE, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, M3_INFRAVISIBLE, CYAN));
	_monsters.push_back(new Monster("electric eel", S_EEL, 7, 10, -3, 0, 0, (G_GENO | G_NOGEN), Attack(AT_BITE, AD_ELEC, 4, 6), Attack(AT_TUCH, AD_WRAP, 0, 0), Attack(), Attack(), Attack(), Attack(), 200, 250, 0, MS_SILENT, MZ_HUGE, MR_ELEC, MR_ELEC, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_SLITHY | M1_NOLIMBS | M1_CARNIVORE | M1_OVIPAROUS | M1_NOTAKE, M2_HOSTILE, M3_INFRAVISIBLE, BOLD_BLUE));
	_monsters.push_back(new Monster("kraken", S_EEL, 20, 3, 6, 0, -3, (G_GENO | G_NOGEN), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_HUGS, AD_WRAP, 2, 6), Attack(AT_BITE, AD_PHYS, 5, 4), Attack(), Attack(), 1800, 1000, 0, MS_SILENT, MZ_HUGE, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_NOPOLY | M2_HOSTILE | M2_STRONG, M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("newt", S_LIZARD, 0, 6, 8, 0, 0, (G_GENO | 5), Attack(AT_BITE, AD_PHYS, 1, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 20, 0, MS_SILENT, MZ_TINY, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, BOLD_YELLOW));
	_monsters.push_back(new Monster("gecko", S_LIZARD, 1, 6, 8, 0, 0, (G_GENO | 5), Attack(AT_BITE, AD_PHYS, 1, 3), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 20, 0, MS_SQEEK, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, GREEN));
	_monsters.push_back(new Monster("iguana", S_LIZARD, 2, 6, 7, 0, 0, (G_GENO | 5), Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 30, 30, 0, MS_SILENT, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("baby crocodile", S_LIZARD, 3, 6, 7, 0, 0, G_GENO, Attack(AT_BITE, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 200, 200, 0, MS_SILENT, MZ_MEDIUM, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("lizard", S_LIZARD, 5, 6, 6, 10, 0, (G_GENO | 5), Attack(AT_BITE, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 10, 40, 0, MS_SILENT, MZ_TINY, MR_STONE, MR_STONE, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_HOSTILE, 0, GREEN));
	_monsters.push_back(new Monster("chameleon", S_LIZARD, 6, 5, 6, 10, 0, (G_GENO | 2), Attack(AT_BITE, AD_PHYS, 4, 2), Attack(), Attack(), Attack(), Attack(), Attack(), 100, 100, 0, MS_SILENT, MZ_TINY, 0, 0, M1_ANIMAL | M1_NOHANDS | M1_CARNIVORE, M2_NOPOLY | M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("crocodile", S_LIZARD, 6, 9, 5, 0, 0, (G_GENO | 1), Attack(AT_BITE, AD_PHYS, 4, 2), Attack(AT_CLAW, AD_PHYS, 1, 12), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_SILENT, MZ_LARGE, 0, 0, M1_SWIM | M1_AMPHIBIOUS | M1_ANIMAL | M1_THICK_HIDE | M1_NOHANDS | M1_OVIPAROUS | M1_CARNIVORE, M2_STRONG | M2_HOSTILE, 0, YELLOW));
	_monsters.push_back(new Monster("salamander", S_LIZARD, 8, 12, -1, 0, -9, (G_HELL | 1), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(AT_TUCH, AD_FIRE, 1, 6), Attack(AT_HUGS, AD_PHYS, 2, 6), Attack(AT_HUGS, AD_FIRE, 3, 6), Attack(), Attack(), 1500, 400, 0, MS_MUMBLE, MZ_HUMAN, MR_SLEEP | MR_FIRE, MR_FIRE, M1_HUMANOID | M1_SLITHY | M1_THICK_HIDE | M1_POIS, M2_STALK | M2_HOSTILE | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, BOLD_RED));
	_monsters.push_back(new Monster("long worm tail", S_WORM_TAIL, 0, 0, 0, 0, 0, (G_NOGEN | G_NOCORPSE | G_UNIQ), Attack(), Attack(), Attack(), Attack(), Attack(), Attack(), 0, 0, 0, 0, 0, 0, 0, 0L, M2_NOPOLY, 0, YELLOW));
	_monsters.push_back(new Monster("archeologist", S_HUMAN, 10, 12, 10, 1, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_TUNNEL | M1_NEEDPICK | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("barbarian", S_HUMAN, 10, 12, 10, 1, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("caveman", S_HUMAN, 10, 12, 10, 0, 1, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("cavewoman", S_HUMAN, 10, 12, 10, 0, 1, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("healer", S_HUMAN, 10, 12, 10, 1, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("knight", S_HUMAN, 10, 12, 10, 1, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("monk", S_HUMAN, 10, 12, 10, 2, 0, G_NOGEN, Attack(AT_CLAW, AD_PHYS, 1, 8), Attack(AT_KICK, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_HERBIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT | M2_MALE, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("priest", S_HUMAN, 10, 12, 10, 2, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_MALE | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("priestess", S_HUMAN, 10, 12, 10, 2, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("ranger", S_HUMAN, 10, 12, 10, 2, -3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("rogue", S_HUMAN, 10, 12, 10, 1, -3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_GREEDY | M2_JEWELS | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("samurai", S_HUMAN, 10, 12, 10, 1, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("valkyrie", S_HUMAN, 10, 12, 10, 1, -1, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH | MR_COLD, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_FEMALE | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("wizard", S_HUMAN, 10, 12, 10, 3, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("Lord Carnarvon", S_HUMAN, 20, 12, 0, 30, 20, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_TUNNEL | M1_NEEDPICK | M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Pelias", S_HUMAN, 20, 12, 0, 30, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Shaman Karnov", S_HUMAN, 20, 12, 0, 30, 20, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Hippocrates", S_HUMAN, 20, 12, 0, 40, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("King Arthur", S_HUMAN, 20, 12, 0, 40, 20, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Grand Master", S_HUMAN, 25, 12, 0, 70, 0, (G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_PHYS, 4, 10), Attack(AT_KICK, AD_PHYS, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_HUMANOID | M1_SEE_INVIS | M1_HERBIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_NASTY | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("Arch Priest", S_HUMAN, 25, 12, 7, 70, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 4, 10), Attack(AT_KICK, AD_PHYS, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(AT_MAGC, AD_CLRC, 2, 8), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_FIRE | MR_ELEC | MR_SLEEP | MR_POISON, 0, M1_HUMANOID | M1_SEE_INVIS | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("Orion", S_HUMAN, 20, 12, 0, 30, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE | M1_SEE_INVIS | M1_SWIM | M1_AMPHIBIOUS, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISION | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Master of Thieves", S_HUMAN, 20, 12, 0, 30, -20, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_SAMU, 2, 4), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_GREEDY | M2_JEWELS | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Lord Sato", S_HUMAN, 20, 12, 0, 30, 20, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_MALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Norn", S_HUMAN, 20, 12, 0, 80, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH | MR_COLD, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_FEMALE | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Neferet the Green", S_HUMAN, 20, 12, 0, 60, 0, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_MAGC, AD_SPEL, 2, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_LEADER, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_FEMALE | M2_PNAME | M2_PEACEFUL | M2_STRONG | M2_COLLECT | M2_MAGIC, M3_CLOSE | M3_INFRAVISIBLE, GREEN));
	_monsters.push_back(new Monster("Minion of Huhetotl", S_DEMON, 16, 12, -2, 75, -14, (G_NOCORPSE | G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 8, 4), Attack(AT_WEAP, AD_PHYS, 4, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_LARGE, MR_FIRE | MR_POISON | MR_STONE, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_STALK | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_COLLECT, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISION | M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("Thoth Amon", S_HUMAN, 16, 12, 0, 10, -14, (G_NOGEN | G_UNIQ | G_NOCORPSE), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_SAMU, 1, 4), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON | MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_STRONG | M2_MALE | M2_STALK | M2_HOSTILE | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Chromatic Dragon", S_DRAGON, 16, 12, 0, 30, -14, (G_NOGEN | G_UNIQ), Attack(AT_BREA, AD_RBRE, 6, 8), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_SAMU, 2, 8), Attack(AT_BITE, AD_PHYS, 4, 8), Attack(AT_BITE, AD_PHYS, 4, 8), Attack(AT_STNG, AD_PHYS, 1, 6), 4500, 1700, 0, MS_NEMESIS, MZ_GIGANTIC, MR_FIRE | MR_COLD | MR_SLEEP | MR_DISINT | MR_ELEC | MR_POISON | MR_ACID | MR_STONE, MR_FIRE | MR_COLD | MR_SLEEP | MR_DISINT | MR_ELEC | MR_POISON | MR_STONE, M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_HOSTILE | M2_FEMALE | M2_STALK | M2_STRONG | M2_NASTY | M2_GREEDY | M2_JEWELS | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Cyclops", S_GIANT, 18, 12, 0, 0, -15, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 4, 8), Attack(AT_WEAP, AD_PHYS, 4, 8), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), Attack(), 1900, 700, 0, MS_NEMESIS, MZ_HUGE, MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_GIANT | M2_STRONG | M2_ROCKTHROW | M2_STALK | M2_HOSTILE | M2_NASTY | M2_MALE | M2_JEWELS | M2_COLLECT, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISION | M3_INFRAVISIBLE, WHITE));
	_monsters.push_back(new Monster("Ixoth", S_DRAGON, 15, 12, -1, 20, -14, (G_NOGEN | G_UNIQ), Attack(AT_BREA, AD_FIRE, 8, 6), Attack(AT_BITE, AD_PHYS, 4, 8), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_PHYS, 2, 4), Attack(AT_CLAW, AD_SAMU, 2, 4), Attack(), 4500, 1600, 0, MS_NEMESIS, MZ_GIGANTIC, MR_FIRE | MR_STONE, MR_FIRE, M1_FLY | M1_THICK_HIDE | M1_NOHANDS | M1_CARNIVORE | M1_SEE_INVIS, M2_NOPOLY | M2_PNAME | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_STALK | M2_GREEDY | M2_JEWELS | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("Master Kaen", S_HUMAN, 25, 12, -10, 10, -20, (G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_PHYS, 16, 2), Attack(AT_CLAW, AD_PHYS, 16, 2), Attack(AT_MAGC, AD_CLRC, 0, 0), Attack(AT_CLAW, AD_SAMU, 1, 4), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON | MR_STONE, MR_POISON, M1_HUMANOID | M1_HERBIVORE | M1_SEE_INVIS, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_HOSTILE | M2_STRONG | M2_NASTY | M2_STALK | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Nalzok", S_DEMON, 16, 12, -2, 85, -127, (G_NOGEN | G_UNIQ | G_NOCORPSE), Attack(AT_WEAP, AD_PHYS, 8, 4), Attack(AT_WEAP, AD_PHYS, 4, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_LARGE, MR_FIRE | MR_POISON | MR_STONE, 0, M1_FLY | M1_SEE_INVIS | M1_POIS, M2_NOPOLY | M2_DEMON | M2_PNAME | M2_HOSTILE | M2_STRONG | M2_STALK | M2_NASTY | M2_COLLECT, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISION | M3_INFRAVISIBLE, RED));
	_monsters.push_back(new Monster("Scorpius", S_SPIDER, 15, 12, 10, 0, -15, (G_NOGEN | G_UNIQ), Attack(AT_CLAW, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(AT_STNG, AD_DISE, 1, 4), Attack(), Attack(), Attack(), 750, 350, 0, MS_NEMESIS, MZ_HUMAN, MR_POISON | MR_STONE, MR_POISON, M1_ANIMAL | M1_NOHANDS | M1_OVIPAROUS | M1_POIS | M1_CARNIVORE, M2_NOPOLY | M2_PNAME | M2_HOSTILE | M2_STRONG | M2_STALK | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU, MAGENTA));
	_monsters.push_back(new Monster("Master Assassin", S_HUMAN, 15, 12, 0, 30, 18, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_DRST, 2, 6), Attack(AT_WEAP, AD_PHYS, 2, 8), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_HUMAN, MR_NO_ELBERETH | MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_HOSTILE | M2_STALK | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Ashikaga Takauji", S_HUMAN, 15, 12, 0, 40, -13, (G_NOGEN | G_UNIQ | G_NOCORPSE), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_WEAP, AD_PHYS, 2, 6), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_HUMAN, MR_NO_ELBERETH | MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PNAME | M2_HOSTILE | M2_STRONG | M2_STALK | M2_NASTY | M2_MALE | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Lord Surtur", S_GIANT, 15, 12, 2, 50, 12, (G_NOGEN | G_UNIQ), Attack(AT_WEAP, AD_PHYS, 2, 10), Attack(AT_WEAP, AD_PHYS, 2, 10), Attack(AT_CLAW, AD_SAMU, 2, 6), Attack(), Attack(), Attack(), 2250, 850, 0, MS_NEMESIS, MZ_HUGE, MR_FIRE | MR_STONE, MR_FIRE, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_GIANT | M2_MALE | M2_PNAME | M2_HOSTILE | M2_STALK | M2_STRONG | M2_NASTY | M2_ROCKTHROW | M2_JEWELS | M2_COLLECT, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISION | M3_INFRAVISIBLE, MAGENTA));
	_monsters.push_back(new Monster("Dark One", S_HUMAN, 15, 12, 0, 80, -10, (G_NOGEN | G_UNIQ | G_NOCORPSE), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_CLAW, AD_SAMU, 1, 4), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), 1450, 400, 0, MS_NEMESIS, MZ_HUMAN, MR_NO_ELBERETH | MR_STONE, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_STRONG | M2_HOSTILE | M2_STALK | M2_NASTY | M2_COLLECT | M2_MAGIC, M3_WANTSARTI | M3_WAITFORU | M3_INFRAVISIBLE, BLACK));
	_monsters.push_back(new Monster("student", S_HUMAN, 5, 12, 10, 10, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_TUNNEL | M1_NEEDPICK | M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("chieftain", S_HUMAN, 5, 12, 10, 10, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("neanderthal", S_HUMAN, 5, 12, 10, 10, 1, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 2, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("attendant", S_HUMAN, 5, 12, 10, 10, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH | MR_POISON, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("page", S_HUMAN, 5, 12, 10, 10, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("abbot", S_HUMAN, 5, 12, 10, 20, 0, G_NOGEN, Attack(AT_CLAW, AD_PHYS, 8, 2), Attack(AT_KICK, AD_STUN, 3, 2), Attack(AT_MAGC, AD_CLRC, 0, 0), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_HERBIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("acolyte", S_HUMAN, 5, 12, 10, 20, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_MAGC, AD_CLRC, 0, 0), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("hunter", S_HUMAN, 5, 12, 10, 10, -7, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 4), Attack(), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_SEE_INVIS | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISION | M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("thug", S_HUMAN, 5, 12, 10, 10, -3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_GREEDY | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("ninja", S_HUMAN, 5, 12, 10, 10, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_HUMANOID, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_HOSTILE | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("roshi", S_HUMAN, 5, 12, 10, 10, 3, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("warrior", S_HUMAN, 5, 12, 10, 10, -1, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(AT_WEAP, AD_PHYS, 1, 8), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT | M2_FEMALE, M3_INFRAVISIBLE, BOLD_WHITE));
	_monsters.push_back(new Monster("apprentice", S_HUMAN, 5, 12, 10, 30, 0, G_NOGEN, Attack(AT_WEAP, AD_PHYS, 1, 6), Attack(AT_MAGC, AD_SPEL, 0, 0), Attack(), Attack(), Attack(), Attack(), 1450, 400, 0, MS_GUARDIAN, MZ_HUMAN, MR_NO_ELBERETH, 0, M1_HUMANOID | M1_OMNIVORE, M2_NOPOLY | M2_HUMAN | M2_PEACEFUL | M2_STRONG | M2_COLLECT | M2_MAGIC, M3_INFRAVISIBLE, BOLD_WHITE));

	/* create mapping */
	for (int i = 0; i < (int) _monsters.size(); ++i) {
		/* based on symbol & color, -1 if ambigous symbol/color */
		if (_monster_symbol_mapping[_monsters[i]->symbol()][_monsters[i]->color()] == -1)
			_monster_symbol_mapping[_monsters[i]->symbol()][_monsters[i]->color()] = i;
		else
			_monster_symbol_mapping[_monsters[i]->symbol()][_monsters[i]->color()] = -2; // ambigous symbol/color
		/* create mapping based on name */
		_monster_name_mapping[_monsters[i]->name()] = i;
	}
}

void Monster::destroy() {
	for (vector<const Monster*>::const_iterator m = _monsters.begin(); m != _monsters.end(); ++m)
		delete *m;
	_monsters.clear();
}

int Monster::saiphDifficultyMin() {
	return _saiph_difficulty_min;
}

int Monster::saiphDifficultyMax() {
	return _saiph_difficulty_max;
}

const vector<const Monster*>& Monster::monsters() {
	return _monsters;
}

const Monster* Monster::monster(const unsigned char& symbol, int color) {
	if (color < 0 || color > INVERSE_BOLD_WHITE)
		return NULL;
	return monster(_monster_symbol_mapping[symbol][color]);
}

const Monster* Monster::monster(const string& name) {
	map<string, int>::iterator m = _monster_name_mapping.find(name);
	if (m == _monster_name_mapping.end())
		return NULL;
	return monster(m->second);
}

/* public methods */
const string& Monster::name() const {
	return _name;
}
const unsigned char& Monster::symbol() const {
	return _symbol;
}

int Monster::difficulty() const {
	return _difficulty;
}

int Monster::moveRate() const {
	return _move_rate;
}

int Monster::ac() const {
	return _ac;
}

int Monster::magicResistance() const {
	return _magic_resistance;
}

int Monster::alignment() const {
	return _alignment;
}

int Monster::genoFlags() const {
	return _geno_flags;
}

int Monster::weight() const {
	return _weight;
}

int Monster::nutrition() const {
	return _nutrition;
}

int Monster::extension() const {
	return _extension;
}

int Monster::sounds() const {
	return _sounds;
}

int Monster::size() const {
	return _size;
}

int Monster::resistances() const {
	return _resistances;
}

int Monster::resistancesConferred() const {
	return _resistances_conferred;
}

int Monster::m1() const {
	return _m1;
}

int Monster::m2() const {
	return _m2;
}

int Monster::m3() const {
	return _m3;
}

int Monster::color() const {
	return _color;
}

bool Monster::ignoresElbereth() const {
	return _ignores_elbereth;
}

// note, the corresponding NetHack function also checks AT_WEAP.  We don't, optimistically, since most weapon-using monsters don't have missiles.  Check ranged() on an individual monster instead.
bool Monster::rangedAttack() const {
	for (vector<Attack>::const_iterator a = _attacks.begin(); a != _attacks.end(); ++a)
		if (a->type() == AT_MAGC || ((1 << a->type()) & ((1 << AT_SPIT) | (1 << AT_BREA) | (1 << AT_GAZE))))
			return true;

	return false;
}

const vector<Attack>& Monster::attacks() const {
	return _attacks;
}

int Monster::saiphDifficulty() const {
	return _saiph_difficulty;
}

/* private static methods */
const Monster* Monster::monster(int id) {
	if (id < 0 || id > (int) _monsters.size())
		return NULL;
	return _monsters[id];
}
